@@ -14,17 +14,14 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-
-from .quantization import Quantization
-from .pruning import Pruning
+"""Intel® Neural Compressor: An open-source Python library supporting popular model compression techniques."""
 from .benchmark import Benchmark
 from .version import __version__
 from .contrib import *
-from .model.model import export
 # we need to set a global 'NA' backend, or Model can't be used
-from .utils.utility import set_backend
+from .utils.utility import set_random_seed, set_tensorboard, set_workspace
 from .utils import options
 from .conf.config import conf
 from .conf.pythonic_config import config
-
-set_backend('NA')
+from .config import DistillationConfig, PostTrainingQuantConfig, \
+                    WeightPruningConfig, QuantizationAwareTrainingConfig
