@@ -24,27 +24,29 @@ from collections.abc import KeysView
 import yaml
 import numpy as np
 from packaging.version import Version
+from importlib.util import find_spec
 from neural_compressor.adaptor.adaptor import adaptor_registry, Adaptor
 from neural_compressor.adaptor.query import QueryBackendCapability
+from neural_compressor.adaptor.ox_utils.util import PROVIDERS, ONNXRT_BACKENDS
 from neural_compressor.utils.utility import LazyImport, dump_elapsed_time, \
                                             GLOBAL_STATE, MODE
 from neural_compressor.utils.utility import Statistics
 from neural_compressor.experimental.data.dataloaders.base_dataloader import BaseDataLoader
 from neural_compressor.conf.dotdict import deep_get
+from neural_compressor.utils.utility import CpuInfo
 import math
 import sys
 
 onnx = LazyImport("onnx")
 ort = LazyImport("onnxruntime")
-if sys.version_info < (3,10): # pragma: no cover
-    ort_ext = LazyImport("onnxruntime_extensions")
 ONNXRT152_VERSION = Version("1.5.2")
 ONNXRT170_VERSION = Version("1.7.0")
 ONNXRT112_VERSION = Version("1.12.0")
 
-logger = logging.getLogger()
+logger = logging.getLogger("neural_compressor")
 
-class ONNXRTAdaptor(Adaptor):
+@adaptor_registry
+class ONNXRUNTIMEAdaptor(Adaptor):
     """The ONNXRT adaptor layer, do onnx-rt quantization, calibration, inspect layer tensors.
 
     Args:
@@ -58,24 +60,62 @@ class ONNXRTAdaptor(Adaptor):
         self.device = framework_specific_info["device"]
         self.static = framework_specific_info["approach"] == "post_training_static_quant"
         self.dynamic = framework_specific_info["approach"] == "post_training_dynamic_quant"
-        self.backend = framework_specific_info["backend"]
+        self.backend = PROVIDERS[framework_specific_info["backend"]]
+
+        if self.backend not in ort.get_all_providers():
+            logger.warning("{} backend is not supported in current environment, "
+                "supported backends: {}".format(ONNXRT_BACKENDS[self.backend],
+                [ONNXRT_BACKENDS[i] for i in ort.get_all_providers()]))
+
+        if (not self.dynamic and "format" in framework_specific_info and \
+            framework_specific_info["format"].lower() == 'qdq') or \
+            self.backend == 'TensorrtExecutionProvider':
+            self.query_handler = ONNXRTQuery(local_config_file=os.path.join(
+                os.path.dirname(__file__), "onnxrt_qdq.yaml"))
+            self.format = "qdq"
+        else:
+            if not self.dynamic:
+                self.query_handler = ONNXRTQuery(local_config_file=os.path.join(
+                    os.path.dirname(__file__), "onnxrt_qlinear.yaml"))
+                self.format = "qlinearops"
+            else:
+                self.query_handler = ONNXRTQuery(local_config_file=os.path.join(
+                    os.path.dirname(__file__), "onnxrt_integer.yaml"))
+                self.format = "integerops"
+                if "format" in framework_specific_info and \
+                    framework_specific_info["format"].lower() == 'qdq':
+                    logger.warning("Dynamic approach doesn't support QDQ format.")
+ 
         self.work_space = framework_specific_info["workspace_path"]
         self.graph_optimization = framework_specific_info["graph_optimization"]
         self.recipes = deep_get(framework_specific_info, 'recipes', {})
         self.reduce_range = framework_specific_info["reduce_range"] if \
-            "reduce_range" in framework_specific_info else None
+            "reduce_range" in framework_specific_info else not CpuInfo().vnni
         self.benchmark = (GLOBAL_STATE.STATE == MODE.BENCHMARK)
         os.makedirs(self.work_space, exist_ok=True)
         self.pre_optimized_model = None
         self.quantizable_op_types = []
         self.query_handler_ext = None
+        if framework_specific_info["approach"] == "post_training_auto_quant" and \
+            self.format != "integerops":
+            self.query_handler_ext = ONNXRTQuery(local_config_file=os.path.join(
+                os.path.dirname(__file__), "onnxrt_integer.yaml"))
+
         for precision in self.query_handler.get_precisions():
             if precision != 'fp32':
-                if self.device == 'cpu' and precision == 'fp16':
-                    continue
                 self.quantizable_op_types += \
                     self.query_handler.get_op_types_by_precision(precision=precision)
  
+        if self.backend == 'TensorrtExecutionProvider':
+            from neural_compressor import options
+            options.onnxrt.qdq_setting.AddQDQPairToWeight = True
+            options.onnxrt.qdq_setting.DedicatedQDQPair = True
+            options.onnxrt.graph_optimization.level = 'DISABLE_ALL'
+            options.onnxrt.qdq_setting.OpTypesToExcludeOutputQuantizatioin = \
+                ['Conv', 'Gemm', 'Add', 'MatMul']
+            self.static = True
+            self.dynamic = False
+
         self.evaluate_nums = 0
 
         self.fp32_results = []
@@ -110,17 +150,13 @@ class ONNXRTAdaptor(Adaptor):
         if model.model.opset_import[0].version < 11: # pragma: no cover
             logger.warning("Quantize input needs model opset 11 or newer.")
         from neural_compressor.adaptor.ox_utils.util import QuantizationMode
-        if self.backend in ["qlinearops", "qoperator"]:
-            backend = QuantizationMode.QLinearOps
-            if self.backend == "qlinearops":
-                logger.warning("onnxrt_qlinearops uses the same model representation format as "
-                               "onnxrt_qoperator. Recommended to use onnxrt_qoperator to align "
-                               "with ONNXRUNTIME QuantFormat")
-        elif self.backend == "qdq":
+        if self.format == "qlinearops":
+            format = QuantizationMode.QLinearOps
+        elif self.format == "qdq":
             assert ort_version >= ONNXRT170_VERSION, 'QDQ mode needs onnxruntime1.7.0 or newer'
-            backend = "qdq"
+            format = "qdq"
         else:
-            backend = QuantizationMode.IntegerOps
+            format = QuantizationMode.IntegerOps
 
         self.quantizable_ops = self._query_quantizable_ops(model.model)
         tmp_model = copy.deepcopy(model)
@@ -143,7 +179,7 @@ class ONNXRTAdaptor(Adaptor):
                             break
                     tmp_iterations = int(math.ceil(calib_sampling_size / calib_batch_size))
                     data_loader.batch(calib_batch_size)
-                    quantize_params = self._get_quantize_params(tmp_model.model, data_loader, \
+                    quantize_params = self._get_quantize_params(tmp_model, data_loader, \
                                                                 quantize_config, tmp_iterations)
                 except Exception as e:  # pragma: no cover
                     if 'Got invalid dimensions for input' in str(e):
@@ -154,8 +190,8 @@ class ONNXRTAdaptor(Adaptor):
                         "Fail to forward with batch size={}, set to {} now.".
                         format(batch_size, 1))
                     data_loader.batch(1)
-                    quantize_params = self._get_quantize_params(tmp_model.model, data_loader, \
-                                                                quantize_config, calib_sampling_size)
+                    quantize_params = self._get_quantize_params(tmp_model, data_loader, \
+                                              quantize_config, calib_sampling_size)
             else:  # pragma: no cover
                 if hasattr(data_loader, 'batch_size') and \
                   calib_sampling_size % data_loader.batch_size != 0:
@@ -165,15 +201,15 @@ class ONNXRTAdaptor(Adaptor):
                         "So the real sampling size is {}.".
                         format(calib_sampling_size, data_loader.batch_size,
                                data_loader.batch_size * iterations))
-                quantize_params = self._get_quantize_params(tmp_model.model, data_loader, \
-                                                            quantize_config, iterations)
+                quantize_params = self._get_quantize_params(tmp_model, data_loader, \
+                                          quantize_config, iterations)
         else:
             quantize_params = None
         self.quantize_params = quantize_params
         from neural_compressor.adaptor.ox_utils.quantizer import Quantizer
-        quantizer = Quantizer(tmp_model.model,
+        quantizer = Quantizer(copy.deepcopy(model),
             quantize_config,
-            backend,
+            format,
             self.static,
             quantize_params,
             self.quantizable_op_types,
@@ -205,7 +241,8 @@ class ONNXRTAdaptor(Adaptor):
         fwk_info = {}
         fwk_info['approach'] = "post_training_static_quant" if self.static else \
                                                         "post_training_dynamic_quant"
-        fwk_info['backend'] = self.backend
+        fwk_info['format'] = self.format
+        fwk_info['backend'] = ONNXRT_BACKENDS[self.backend]
         fwk_info['workspace_path'] = self.work_space
         fwk_info['graph_optimization'] = self.graph_optimization
         fwk_info['device'] = self.device
@@ -233,20 +270,20 @@ class ONNXRTAdaptor(Adaptor):
             logger.warning("Quantize input needs model opset 11 or newer.")
 
         from neural_compressor.adaptor.ox_utils.util import QuantizationMode
-        if self.backend in ["qlinearops", "qoperator"]:
-            backend = QuantizationMode.QLinearOps
-        elif self.backend == "qdq":
+        if self.format in ["qlinearops"]:
+            format = QuantizationMode.QLinearOps
+        elif self.format == "qdq":
             assert ort_version >= ONNXRT170_VERSION, 'QDQ mode needs onnxruntime1.7.0 or newer'
-            backend = self.backend
+            format = self.format
         else:
-            backend = QuantizationMode.IntegerOps
+            format = QuantizationMode.IntegerOps
         from neural_compressor.adaptor.ox_utils.quantizer import Quantizer
         self.quantizable_ops = self._query_quantizable_ops(model.model)
         quantize_params, tune_cfg = self._parse_qconfig(q_config)
         quantize_config = self._cfg_to_quantize_config(tune_cfg)
         quantizer = Quantizer(model.model,
             quantize_config,
-            backend,
+            format,
             self.static,
             quantize_params,
             self.quantizable_op_types,
@@ -340,9 +377,10 @@ class ONNXRTAdaptor(Adaptor):
         augment = ONNXRTAugment(model, \
                   data_loader, self.quantizable_op_types, \
                   black_nodes=black_nodes, white_nodes=white_nodes, \
-                  iterations=list(range(0, quantize_config['calib_iteration'])))
+                  iterations=list(range(0, quantize_config['calib_iteration'])),
+                  backend=self.backend, reduce_range=self.reduce_range)
         self.min_max = augment.dump_minmax()
-        quantize_params = augment.dump_calibration()
+        quantize_params = augment.dump_calibration(quantize_config)
         return quantize_params
 
     def inspect_tensor(self, model, dataloader, op_list=[],
@@ -363,7 +401,8 @@ class ONNXRTAdaptor(Adaptor):
             op_list = [item[0] for item in op_list]
         augment = ONNXRTAugment(model, dataloader, [], \
                   iterations=iteration_list,
-                  white_nodes=op_list)
+                  white_nodes=op_list,
+                  backend=self.backend)
         tensors = augment.dump_tensor(activation=(inspect_type!='weight'),
                                       weight=(inspect_type!='activation'))
         if save_to_disk:
@@ -452,22 +491,71 @@ class ONNXRTAdaptor(Adaptor):
                     'ENABLE_ALL': ort.GraphOptimizationLevel.ORT_ENABLE_ALL}
             assert self.graph_optimization.level in optimization_levels, "the optimization \
                                       choices are {}".format(optimization_levels.keys())
- 
+
             level = optimization_levels[self.graph_optimization.level]
         sess_options.graph_optimization_level = level
         sess_options.optimized_model_filepath = os.path.join(self.work_space, \
             "Optimized_model.onnx")
-        if sys.version_info < (3,10): # pragma: no cover 
-            sess_options.register_custom_ops_library(ort_ext.get_library_path())
-        _ = ort.InferenceSession(model.model.SerializeToString(), sess_options)
-        tmp_model = onnx.load(sess_options.optimized_model_filepath)
+        if sys.version_info < (3,10) and find_spec('onnxruntime_extensions'): # pragma: no cover
+            from onnxruntime_extensions import get_library_path
+            sess_options.register_custom_ops_library(get_library_path())
+        if not model.large_size:
+            ort.InferenceSession(model.model.SerializeToString(),
+                                 sess_options,
+                                 providers=[self.backend])
+        elif model.model_path is not None: # pragma: no cover
+            ort.InferenceSession(model.model_path,
+                                 sess_options,
+                                 providers=[self.backend])
+        else: # pragma: no cover 
+            logger.warning('Please use model path instead of onnx model object to quantize')
+
+        tmp_model = onnx.load(sess_options.optimized_model_filepath, load_external_data=False)
+        if model.large_size: # pragma: no cover
+            from onnx.external_data_helper import load_external_data_for_model
+            load_external_data_for_model(tmp_model, os.path.split(model.model_path)[0])
+        model.model_path = sess_options.optimized_model_filepath
         model.model = self._replace_gemm_with_matmul(tmp_model).model \
             if self.graph_optimization.gemm2matmul else tmp_model
         model.model = self._rename_node(model.model)
         model = self._revert_fusedconv(model)
+        if self.backend == 'TensorrtExecutionProvider':
+            model = self._revert_conv_add_fusion(model)
         model = split_shared_bias(model)
         model.topological_sort()
-        self.pre_optimized_model = model
+        self.pre_optimized_model = copy.deepcopy(model)
+
+    def _revert_conv_add_fusion(self, model):
+        from onnx import numpy_helper
+        from neural_compressor.adaptor.ox_utils.util import attribute_to_kwarg
+        add_nodes = []
+        remove_nodes = []
+        for node in model.model.graph.node:
+            if node.op_type == 'Conv' and len(node.input) == 3:
+                bias_tensor = model.get_initializer(node.input[2])
+                bias_array = numpy_helper.to_array(bias_tensor).reshape((-1, 1, 1))
+                model.remove_initializer(bias_tensor)
+                model.add_initializer(numpy_helper.from_array(bias_array, bias_tensor.name))
+                kwargs = {}
+                activation_params = None
+                for attr in node.attribute:
+                    kwargs.update(attribute_to_kwarg(attr))
+                conv = onnx.helper.make_node(
+                    'Conv',
+                    node.input[0:2],
+                    [node.name + '_revert'],
+                    node.name, **kwargs)
+                add = onnx.helper.make_node(
+                    'Add',
+                    [conv.output[0], node.input[2]],
+                    node.output,
+                    node.name + '_add')
+                add_nodes.extend([conv, add])
+
+        model.remove_nodes(remove_nodes)
+        model.add_nodes(add_nodes)
+        model.update()
+        return model
 
     def _revert_fusedconv(self, model):
         from neural_compressor.adaptor.ox_utils.util import attribute_to_kwarg
@@ -604,6 +692,12 @@ class ONNXRTAdaptor(Adaptor):
         exclude_first_quantizable_op = True if 'first_conv_or_matmul_quantization' in \
             self.recipes and not self.recipes['first_conv_or_matmul_quantization'] \
             else False
+        exclude_last_quantizable_op = True if 'last_conv_or_matmul_quantization' in \
+            self.recipes and not self.recipes['last_conv_or_matmul_quantization'] \
+            else False
+        exclude_pre_post_process = True if 'pre_post_process_quantization' in \
+            self.recipes and not self.recipes['pre_post_process_quantization'] \
+            else False
  
         quantizable_optype = set([i.op_type for i in self.pre_optimized_model.nodes()])
         optype_wise = OrderedDict()
@@ -614,28 +708,32 @@ class ONNXRTAdaptor(Adaptor):
             precisions = query.get_precisions()
 
             for precision in precisions:
-                if precision == 'fp16' and self.device == 'cpu' and os.getenv('FORCE_FP16') != '1':
-                    continue
-                if precision in query.get_quantization_capability():
-                    special_config_types = list(query.get_quantization_capability() \
-                        [precision].keys())
-                    default_config = query.get_quantization_capability() \
-                        [precision]['default']
-                else:
-                    special_config_types = {}
-                    default_config = {'weight': {'dtype': precision}, 
-                                      'activation': {'dtype': precision}}
+                # get supported optype for target precision
                 optypes = query.get_op_types_by_precision(precision) if \
                     query.get_op_types_by_precision(precision) != ['*'] else \
                     optype_wise.keys()
+ 
+                if self.backend in query.get_quantization_capability():
+                    configs = query.get_quantization_capability()[self.backend] if \
+                        precision in query.get_quantization_capability() else \
+                        {'default': {'weight': {'dtype': precision}, 'activation': {'dtype': precision}}}
+                else:
+                    continue
+
+                if self.backend == 'TensorrtExecutionProvider' and \
+                    precision not in query.get_fallback_list():
+                    optypes.append('Add')
+ 
                 for op in optypes:
                     if op not in quantizable_optype:
                         continue
-                    if op not in special_config_types:
-                        op_capability = copy.deepcopy(default_config)
+                    if op not in configs:
+                        if 'default' in configs:
+                            op_capability = copy.deepcopy(configs['default'])
+                        else:
+                            continue
                     else:
-                        op_capability = copy.deepcopy(
-                            query.get_quantization_capability()[precision][op])
+                        op_capability = copy.deepcopy(configs[op])
 
                     if precision in ['int8', 'uint8']:
                         if self.static:
@@ -652,16 +750,81 @@ class ONNXRTAdaptor(Adaptor):
                     elif op_capability not in optype_wise[op]:
                         optype_wise[op].append(op_capability)
 
+        first_quantizable_node = []
+        last_quantizable_node = []
+        all_conv_matmul = []
         for _, node in enumerate(self.pre_optimized_model.nodes()):
+            if node.op_type in ['Conv', 'MatMul']:
+                # get first Conv or MatMul node
+                if exclude_first_quantizable_op:
+                    if len(first_quantizable_node) == 0:
+                        first_quantizable_node.append(node.name)
+                
+                # get last Conv or MatMul node
+                if exclude_last_quantizable_op:
+                    if len(last_quantizable_node) != 0:
+                        last_quantizable_node.pop()
+                    last_quantizable_node.append(node.name)
+
+                # get first and last Conv or MatMul node
+                if exclude_pre_post_process:
+                    if len(first_quantizable_node) == 0:
+                        first_quantizable_node.append(node.name)
+                    if len(last_quantizable_node) != 0:
+                        last_quantizable_node.pop()
+                    last_quantizable_node.append(node.name)
+                    all_conv_matmul.append(node)
+
+        for _, node in enumerate(self.pre_optimized_model.nodes()):
+            # for TRT EP, only insert Q/DQ to inputs of Add nodes followed by ReduceMean
+            if node.op_type == 'Add' and self.backend == 'TensorrtExecutionProvider':
+                children = self.pre_optimized_model.get_children(node)
+                if 'ReduceMean' not in [i.op_type for i in children]:
+                    op_wise.update({(node.name, node.op_type): 
+                        [{'weight': {'dtype': 'fp32'}, 'activation': {'dtype': 'fp32'}}]})
+                continue
+
             if node.op_type in optype_wise:
-                if exclude_first_quantizable_op and node.op_type in ['Conv', 'MatMul']:
-                    exclude_first_quantizable_op = False
+                if (exclude_first_quantizable_op and node.name in first_quantizable_node) \
+                     or (exclude_last_quantizable_op and node.name in last_quantizable_node):
                     tmp_cfg = copy.deepcopy(optype_wise[node.op_type])
                     tmp_cfg = list(filter(lambda x:'quant_mode' not in x['activation'], tmp_cfg))
                     op_wise.update({(node.name, node.op_type): tmp_cfg})
                     continue
                 op_wise.update(
                     {(node.name, node.op_type): copy.deepcopy(optype_wise[node.op_type])})
+
+        if exclude_pre_post_process:
+            from collections import deque
+            
+            # get nodes between first quantizable node and last quantizable node
+            backbone_queue = deque(last_quantizable_node)
+            backbone_nodes = self.pre_optimized_model.get_nodes_chain(backbone_queue, first_quantizable_node)
+
+            # get extra Conv or MatMul nodes not between first quantizable node and last quantizable node
+            backbone_queue_extra = deque()
+            for conv_or_matmul in all_conv_matmul:
+                if conv_or_matmul.name not in backbone_nodes:
+                    backbone_queue_extra.append(conv_or_matmul.name)
+                    backbone_nodes = self.pre_optimized_model.get_nodes_chain(backbone_queue_extra, 
+                                                    first_quantizable_node, backbone_nodes)
+            backbone_nodes += [i for i in first_quantizable_node]
+
+            for _, node in enumerate(self.pre_optimized_model.nodes()):
+                if node.op_type in optype_wise:
+                    # nodes not in backbone are not quantized
+                    if node.name not in backbone_nodes:
+                        tmp_cfg = copy.deepcopy(optype_wise[node.op_type])
+                        tmp_cfg = list(filter(lambda x:'quant_mode' not in x['activation'], tmp_cfg))
+                        op_wise.update({(node.name, node.op_type): tmp_cfg})
+                        continue
+                    if (node.name, node.op_type) in op_wise:
+                        op_wise.update(
+                            {(node.name, node.op_type): copy.deepcopy(op_wise[(node.name, node.op_type)])})
+                    else: # pragma: no cover
+                        op_wise.update(
+                            {(node.name, node.op_type): copy.deepcopy(optype_wise[node.op_type])})
+
         return {'optypewise': optype_wise, 'opwise': op_wise}
 
     def _cfg_to_quantize_config(self, tune_cfg):
@@ -672,6 +835,8 @@ class ONNXRTAdaptor(Adaptor):
 
         from onnx import onnx_pb as onnx_proto
         for _, op in enumerate(self.quantizable_ops):
+            if (op.name, op.op_type) not in tune_cfg['op']:
+                continue
             if tune_cfg['op'][(op.name, op.op_type)]['activation']['dtype'] in \
                 self.query_handler.get_fallback_list():
                 quantize_config[op.name] = \
@@ -724,18 +889,30 @@ class ONNXRTAdaptor(Adaptor):
         Returns:
             (float) evaluation results. acc, f1 e.g.
         """
+        if input_graph.large_size: # pragma: no cover
+            onnx.save_model(input_graph.model,
+                            self.work_space + 'eval.onnx',
+                            save_as_external_data=True,
+                            all_tensors_to_one_file=True,
+                            location="weights.pb",
+                            convert_attribute=False)
         sess_options = ort.SessionOptions()
+        if self.backend == 'TensorrtExecutionProvider':
+            sess_options.graph_optimization_level = ort.GraphOptimizationLevel.ORT_DISABLE_ALL 
         if measurer:
             # https://github.com/microsoft/onnxruntime/issues/7347
             cores_per_instance = int(os.environ.get('CORES_PER_INSTANCE'))
             assert cores_per_instance > 0, "benchmark cores_per_instance should greater than 0"
             sess_options.intra_op_num_threads = cores_per_instance
-        try:
-            session = ort.InferenceSession(input_graph.model.SerializeToString(), sess_options)
-        except:
-            if sys.version_info < (3,10): # pragma: no cover 
-                sess_options.register_custom_ops_library(ort_ext.get_library_path())
-            session = ort.InferenceSession(input_graph.model.SerializeToString(), sess_options)
+        if sys.version_info < (3,10) and find_spec('onnxruntime_extensions'): # pragma: no cover
+            from onnxruntime_extensions import get_library_path
+            sess_options.register_custom_ops_library(get_library_path())
+        session = ort.InferenceSession(self.work_space + 'eval.onnx',
+                                       sess_options,
+                                       providers=[self.backend]) if input_graph.large_size else \
+                  ort.InferenceSession(input_graph.model.SerializeToString(),
+                                       sess_options,
+                                       providers=[self.backend])
         results = []
         if metrics:
             for metric in metrics:
@@ -820,10 +997,10 @@ class ONNXRTAdaptor(Adaptor):
     def diagnosis_helper(self, fp32_model, int8_model, tune_cfg=None, save_path=None):
         from neural_compressor.utils.utility import dump_data_to_local
         from neural_compressor.adaptor.ox_utils.util import find_by_name
-        if self.backend in ["qlinearops", "qoperator"]:
+        if self.format == "qlinearops":
             supported_optype = ['Conv', 'MatMul', 'Concat', 'Attention', 'FusedConv',
                 'Add', 'Mul', 'LeakyRelu', 'Sigmoid', 'GlobalAveragePool', 'AveragePool']
-        elif self.backend == "qdq":
+        elif self.format == "qdq":
             supported_optype = ['Conv', 'MatMul', 'Concat', 'Attention', 'FusedConv',
                 'LeakyRelu', 'Sigmoid', 'GlobalAveragePool', 'AveragePool']
         else:
@@ -857,7 +1034,7 @@ class ONNXRTAdaptor(Adaptor):
 
 
 @adaptor_registry
-class ONNXRT_QLinearOpsAdaptor(ONNXRTAdaptor):
+class ONNXRT_QLinearOpsAdaptor(ONNXRUNTIMEAdaptor):
     """The ONNXRT adaptor layer, do onnx-rt quantization, calibration, inspect layer tensors.
 
     Args:
@@ -865,30 +1042,10 @@ class ONNXRT_QLinearOpsAdaptor(ONNXRTAdaptor):
     """
 
     def __init__(self, framework_specific_info):
-        self.query_handler = ONNXRTQuery(local_config_file=os.path.join(
-            os.path.dirname(__file__), "onnxrt_qlinear.yaml"))
-        self.backend = "qlinearops"
-        super().__init__(framework_specific_info)
-        if framework_specific_info["approach"] == "post_training_auto_quant":
-            self.query_handler_ext = ONNXRTQuery(local_config_file=os.path.join(
-                os.path.dirname(__file__), "onnxrt_integer.yaml"))
-
-@adaptor_registry
-class ONNXRT_QOperatorAdaptor(ONNXRTAdaptor):
-    """The ONNXRT adaptor layer, do onnx-rt quantization, calibration, inspect layer tensors.
-
-    Args:
-        framework_specific_info (dict): framework specific configuration for quantization.
-    """
-
-    def __init__(self, framework_specific_info):
-        self.query_handler = ONNXRTQuery(local_config_file=os.path.join(
-            os.path.dirname(__file__), "onnxrt_qlinear.yaml"))
-        self.backend = "qlinearops"
         super().__init__(framework_specific_info)
 
 @adaptor_registry
-class ONNXRT_IntegerOpsAdaptor(ONNXRTAdaptor):
+class ONNXRT_IntegerOpsAdaptor(ONNXRUNTIMEAdaptor):
     """The ONNXRT adaptor layer, do onnx-rt quantization, calibration, inspect layer tensors.
 
     Args:
@@ -896,13 +1053,10 @@ class ONNXRT_IntegerOpsAdaptor(ONNXRTAdaptor):
     """
 
     def __init__(self, framework_specific_info):
-        self.query_handler = ONNXRTQuery(local_config_file=os.path.join(
-            os.path.dirname(__file__), "onnxrt_integer.yaml"))
-        self.backend = "integerops"
         super().__init__(framework_specific_info)
 
 @adaptor_registry
-class ONNXRT_QDQAdaptor(ONNXRTAdaptor):
+class ONNXRT_QDQAdaptor(ONNXRUNTIMEAdaptor):
     """The ONNXRT adaptor layer, do onnx-rt quantization, calibration, inspect layer tensors.
 
     Args:
@@ -910,13 +1064,7 @@ class ONNXRT_QDQAdaptor(ONNXRTAdaptor):
     """
 
     def __init__(self, framework_specific_info):
-        self.query_handler = ONNXRTQuery(local_config_file=os.path.join(
-            os.path.dirname(__file__), "onnxrt_qdq.yaml"))
-        self.backend = "qdq"
         super().__init__(framework_specific_info)
-        if framework_specific_info["approach"] == "post_training_auto_quant":
-            self.query_handler_ext = ONNXRTQuery(local_config_file=os.path.join(
-                os.path.dirname(__file__), "onnxrt_integer.yaml"))
 
 class ONNXRTQuery(QueryBackendCapability):
 
