@@ -14,6 +14,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Fuse Decomposed InstanceNorm Graph Rewriter."""
 
 import re
 
@@ -27,12 +28,16 @@ from neural_compressor.adaptor.tf_utils.quantize_graph_common import QuantizeGra
 from neural_compressor.utils.utility import dump_elapsed_time
 
 class FuseDecomposedINOptimizer(): # pragma: no cover
+    """Fuse decomposed small ops into InstanceNorm."""
+
     def __init__(self, input_graph_def):
+        """Initilization."""
         self.input_graph_def = input_graph_def
 
     @dump_elapsed_time("Pass FuseDecomposedINOptimizer")
     def do_transformation(self):
-        """Find a group of ops that make up an instance/layer normalization pattern for fusion。
+        """Find a group of ops that make up an instance/layer normalization pattern for fusion.
+
         In some models, the instance normalizatin is performed via a group of individual
         ops instead of using single InstanceNormalization op. This function identifies a
         pattern of instance normalization subgraph which is made of multiple ops and
@@ -49,39 +54,41 @@ class FuseDecomposedINOptimizer(): # pragma: no cover
                      Subgraph for fusion
                      -------------------
           *(input)
-           |    | \____________
-           |    |              \
-           |    |             Mean1                                      FusedOp
-           |    |            /    \                                      -------
-           |    |           /      \                           *(input)  Const  Const
-           |    |          /        \                              \    (gamma) (beta)
-           |    |         /          \                              \     |     /
-           |    |        /           |                _MklFusedInstanceNorm/_MklLayerNorm
-           |    |       /            |
-           \   SquaredDiff  Const    |
-            \      \      /          |
-             \      \    /           |
-              \     Mean0  Const     |
-               \      \    /         |
-                \  AddV2|Add         |
-                 \       \    Const  |
-                  \    Rsqrt (gamma) |
-                   \        \ /      |
-                    \       Mul1     |
-                     \      |   \    |
-                      \     |    \   |
-                       \    |     \  |
-                        \   | Const\ |
-                         \  | (beta)Mul2
-                          \ |    |  /
+           x    x x____________
+           x    x              x
+           x    x             Mean1                                      FusedOp
+           x    x            x    x                                      -------
+           x    x           x      x                           *(input)  Const  Const
+           x    x          x        x                              x    (gamma) (beta)
+           x    x         x          x                              x     x     x
+           x    x        x           x                _MklFusedInstanceNorm/_MklLayerNorm
+           x    x       x            x
+           x   SquaredDiff  Const    x
+            x      x      x          x
+             x      x    x           x
+              x     Mean0  Const     x
+               x      x    x         x
+                x  AddV2|Add         x
+                 x       x    Const  x
+                  x    Rsqrt (gamma) x
+                   x        x x      x
+                    x       Mul1     x
+                     x      x   x    x
+                      x     x    x   x
+                       x    x     x  x
+                        x   x Constx x
+                         x  x (beta)Mul2
+                          x x    x  x
                           Mul0   Sub
-                             \   /
+                             x   x
                          AddV2|Add(output)
         Args:
             input_graph_def: A GraphDef containing a model.
+
         Returns:
             Modified graph with individual ops that made up of instance normalization
             fused to InstanceNormalization.
+
         Raises:
             ValueError: If the graph is badly formed with duplicate node names.
         """
@@ -278,11 +285,14 @@ def node_name_from_input(node_name):
 
 def node_from_map(node_map, name):
     """Pulls a node def from a dictionary for a given name.
+
     Args:
         node_map: Dictionary containing an entry indexed by name for every node.
         name: Identifies the node we want to find.
+
     Returns:
         NodeDef of the node with the given name.
+
     Raises:
         ValueError: If the node isn't present in the dictionary.
     """
@@ -293,10 +303,13 @@ def node_from_map(node_map, name):
 
 def values_from_const(node_def):
     """Extracts the values from a const NodeDef as a numpy ndarray.
+
     Args:
         node_def: Const NodeDef that has the values we want to access.
+
     Returns:
         Numpy ndarray containing the values.
+
     Raises:
         ValueError: If the node isn't a Const.
     """
@@ -309,6 +322,7 @@ def values_from_const(node_def):
     return tensor_value
 
 def valid_reshape_inputs(reshape_in0_ndef, reshape_in1_ndef):
+    """Check if the inputs of the Reshape are valid."""
     if reshape_in0_ndef.op != "Const" or reshape_in1_ndef.op != "Const" \
     or get_const_dim_count(reshape_in0_ndef) != 1:
         return False
@@ -326,6 +340,7 @@ def valid_reshape_inputs(reshape_in0_ndef, reshape_in1_ndef):
     return True
 
 def bypass_reshape(input_node_map, input_name):
+    """Get Reshape input nodes."""
     reshape_ndef = None
     maybe_reshape_ndef = node_from_map(input_node_map, input_name)
     input_ndef = maybe_reshape_ndef
@@ -342,8 +357,10 @@ def bypass_reshape(input_node_map, input_name):
 
 def get_const_dim_count(node_def):
     """Get the number of dimensions for a Const node.
+
     Args:
         node_def: Const NodeDef.
+
     Returns:
         Number of dimensions for the Const node.
     """
