@@ -14,6 +14,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Fuse QuantizedMatMul with Requantize/Dequantize Graph Rewriter."""
 
 import numpy as np
 import tensorflow as tf
@@ -30,9 +31,10 @@ from neural_compressor.adaptor.tf_utils.util import version1_gt_version2, versio
 
 
 class FuseMatMulRequantizeDequantizeTransformer(GraphRewriterBase):
-    """Fuse QuantizedMatMul + Requantize + Dequantize into QuantizedMatMulWithBiasAndDequantize.
-    """
+    """Fuse QuantizedMatMul + Requantize + Dequantize into QuantizedMatMulWithBiasAndDequantize."""
+
     def __init__(self, model, device='cpu'):
+        """Initilization."""
         super().__init__(model)
         self.device = device
         self.graph_analyzer = GraphAnalyzer()
@@ -43,6 +45,7 @@ class FuseMatMulRequantizeDequantizeTransformer(GraphRewriterBase):
         self.eps = 1e-5
 
     def do_transformation(self):
+        """Apply the fusion of QuantizedMatMul + Requantize + Dequantize."""
         fuse_pattern = []
         if tf.version.VERSION in ("1.15.0-up2", "1.15.0-up3") or version1_gt_version2(tf.version.VERSION, '2.1.0'):
             fuse_pattern = [["QuantizedMatMulWithBias"], ['Requantize'], ['Dequantize'], ('Softmax',)]
@@ -184,9 +187,9 @@ class FuseMatMulRequantizeDequantizeTransformer(GraphRewriterBase):
         return self.graph_analyzer.dump_graph()
 
 class FuseMatMulRequantizeTransformer(GraphRewriterBase):
-    """Fuse Quantized MatMul Op with the successor Requantize Op.
-    """
+    """Fuse Quantized MatMul Op with the successor Requantize Op."""
     def __init__(self, model, device='cpu'):
+        """Initilization."""
         super().__init__(model)
         self.device = device
         self.graph_analyzer = GraphAnalyzer()
@@ -196,6 +199,7 @@ class FuseMatMulRequantizeTransformer(GraphRewriterBase):
 
     def do_transformation(self):
         """Fuse the quantized op with the following requantize op.
+
         Returns:
             [graphdef]: the optimized graphdef object
         """
@@ -337,9 +341,10 @@ class FuseMatMulRequantizeTransformer(GraphRewriterBase):
         return self.graph_analyzer.dump_graph()
 
 class FuseMatMulRequantizeDequantizeNewAPITransformer(GraphRewriterBase): # pragma: no cover
-    """Fuse _QuantizedMatMul + Requantize + Dequantize into _QuantizedMatMul.
-    """
+    """Fuse _QuantizedMatMul + Requantize + Dequantize into _QuantizedMatMul."""
+
     def __init__(self, model, device='cpu'):
+        """Initilization."""
         super().__init__(model)
         self.device = device
         self.graph_analyzer = GraphAnalyzer()
@@ -350,6 +355,7 @@ class FuseMatMulRequantizeDequantizeNewAPITransformer(GraphRewriterBase): # prag
         self.eps = 1e-5
 
     def do_transformation(self):
+        """Apply the fusion of QuantizedMatMul + Requantize + Dequantize."""
         fuse_pattern = [["_QuantizedMatMul"], ['Requantize'], ['Dequantize'], ('Softmax',)]
 
         uint8_type = dtypes.quint8.as_datatype_enum
@@ -520,9 +526,10 @@ class FuseMatMulRequantizeDequantizeNewAPITransformer(GraphRewriterBase): # prag
         return self.graph_analyzer.dump_graph()
 
 class FuseMatMulRequantizeNewAPITransformer(GraphRewriterBase):
-    """Fuse newAPI Quantized MatMul Op with the successor Requantize Op.
-    """
+    """Fuse newAPI Quantized MatMul Op with the successor Requantize Op."""
+
     def __init__(self, model, device='cpu'):
+        """Initilization."""
         super().__init__(model)
         self.device = device
         self.graph_analyzer = GraphAnalyzer()
@@ -532,6 +539,7 @@ class FuseMatMulRequantizeNewAPITransformer(GraphRewriterBase):
 
     def do_transformation(self):
         """Fuse the quantized op with the following requantize op.
+
         Returns:
             [graphdef]: the optimized graphdef object
         """
@@ -588,10 +596,16 @@ class FuseMatMulRequantizeNewAPITransformer(GraphRewriterBase):
             min_filter_node = None
             # The Min and Max of non-const weight node are from QuantizeV2's output, not valid nodes.
             # Add check here for excluding this case.
-            if ":2" not in new_node.input[6]:
-                max_filter_node = self.graph_info[new_node.input[6]].node
-            if ":1" not in new_node.input[5]:
-                min_filter_node = self.graph_info[new_node.input[5]].node
+            if len(attr_fused_ops) == 0: # single matmul case
+                if ":2" not in new_node.input[5]:
+                    max_filter_node = self.graph_info[new_node.input[5]].node
+                if ":1" not in new_node.input[4]:
+                    min_filter_node = self.graph_info[new_node.input[4]].node
+            else:
+                if ":2" not in new_node.input[6]:
+                    max_filter_node = self.graph_info[new_node.input[6]].node
+                if ":1" not in new_node.input[5]:
+                    min_filter_node = self.graph_info[new_node.input[5]].node
             last_node = self.graph_info[new_node.input[0]].node
             is_min_first = bool(quantized_node.attr['input_quant_mode'].s == b'MIN_FIRST')
             weight_node = self.graph_info[new_node.input[1]].node
@@ -611,7 +625,8 @@ class FuseMatMulRequantizeNewAPITransformer(GraphRewriterBase):
                     continue
                 min_input_node = min_input_parent_node
                 max_input_node = max_input_parent_node
-            if max_filter_node and min_filter_node and max_filter_node.op == 'Enter': # pragma: no cover
+            if max_filter_node and min_filter_node and min_filter_node.input and \
+               max_filter_node.input and max_filter_node.op == 'Enter': # pragma: no cover
                 min_filter_parent_name = Helper.node_name_from_input(min_filter_node.input[0])
                 max_filter_parent_name = Helper.node_name_from_input(max_filter_node.input[0])
                 min_filter_parent_node = self.graph_info[min_filter_parent_name].node
