@@ -14,10 +14,12 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Insert QDQ pattern Graph Rewriter."""
 
 import copy
 import numpy as np
 from collections import namedtuple
+from tensorflow.core.framework import attr_value_pb2
 from tensorflow.python.framework import dtypes
 from tensorflow.python.framework import tensor_util
 from neural_compressor.utils.utility import dump_elapsed_time
@@ -27,17 +29,10 @@ from neural_compressor.adaptor.tf_utils.graph_util import GraphRewriterHelper as
 import re
 
 class GenerateGraphWithQDQPattern(GraphRewriterBase):
-    """ Insert Q/DQ pairs before quantizable ops.
-
-    Args: model: input model.
-          data: sampling data.
-          device: cpu or gpu
-
-    Return: converted model with QDQ pattern
-    """
-
+    """Insert Q/DQ pairs before quantizable ops."""
     def __init__(self, model, calibration_data, op_wise_config, fake_quant, fp32_ops,
                  bf16_ops, quantized_nodes, device, performance_only, itex_mode):
+        """Initilization."""
         super().__init__(model)
         self.data = calibration_data
         self.op_wise_config = op_wise_config
@@ -62,6 +57,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
 
     @dump_elapsed_time("Pass GenerateGraphWithQDQPattern")
     def do_transformation(self):
+        """Generate the graph with QDQ patterns, this is the first step to do new api quantizaiton."""
         min_max_values = {}
         for i in self.data:
             if i.find('_requant') == -1:
@@ -79,7 +75,9 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
         self.g = GraphAnalyzer()
         self.g.graph = copy.deepcopy(self.model)
         self.graph_info = self.g.parse_graph()
-        
+
+        self.g.get_frame_info()
+
         # insert QDQ pattern for op's input
         for op_name in quantizable_op_names:
             if self._ignore_insert_qdq_pattern(op_name):
@@ -91,8 +89,9 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
                 op_wise_cfg = self.op_wise_config[op_name]
                 is_asymmetric = op_wise_cfg[2]
             if self.graph_info[op_name].node.op == "ConcatV2":
-                self._insert_qdq_pattern_for_concatv2(self.graph_info[op_name].node,
-                                                      is_asymmetric)
+                if not self.itex_mode:
+                    self._insert_qdq_pattern_for_concatv2(self.graph_info[op_name].node,
+                                                        is_asymmetric)
             else:
                 self._insert_qdq_pattern_for_common_ops(self.graph_info[op_name].node,
                                                         is_asymmetric)
@@ -113,20 +112,16 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
 
             computational_node = self.graph_info[computational_node_name].node
             weight_name = computational_node.input[1]
-            weight_node = self.graph_info[weight_name].node
             if re.search(r"\w+:\d+", weight_name):
                 weight_node = self.graph_info[weight_name.rsplit(':', 1)[0]].node
             else:
                 weight_node = self.graph_info[weight_name].node
-            enter_node = None
             if weight_node.op == 'Enter':
                 if self.itex_mode:
                     parent_node = self.graph_info[Helper.node_name_from_input(weight_node.input[0])].node
                     if not parent_node.op == 'Const':
                         continue
-                    else:
-                        enter_node = weight_node
-                        weight_node = parent_node
+                    weight_node = parent_node
                 else:
                     continue
 
@@ -137,10 +132,10 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             else:
                 per_channel = False
                 weight_bit = 7
-            
+
             self._insert_qdq_pattern_for_weight_node(computational_node,
                                                      weight_node,
-                                                     enter_node,
+                                                     weight_name,
                                                      min_max_values,
                                                      per_channel,
                                                      weight_bit,
@@ -178,14 +173,16 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
         return self.g_qdq.dump_graph()
 
     def _check_op_list(self, node_type):
+        """Check if the node_type in the allowed op list."""
         op_list = ("ConcatV2", "Conv2D", "Conv3D", "DepthwiseConv2D", "QuantizeV2", "DepthwiseConv2dNative",
                    "MaxPool", "MaxPool3D", "FusedBatchNormV3", "Requantize", "RequantizePerChannel", "AvgPool", "Pad",
                    "CropAndResize", "Dequantize", "Mean", "MatMul", "BatchMatMul", "BatchMatMulV2",
                    "FakeQuantWithMinMaxVars", "_MklFusedInstanceNorm",
-                   "Conv2DBackpropInput", "Conv3DBackpropInputV2")
+                   "Conv2DBackpropInput", "Conv3DBackpropInputV2", "Sigmoid")
         return any([node_type.find(i) != -1 for i in op_list])
 
     def _find_relu_node(self, node):
+        """Find Relu node algorithm to identify the positive input."""
         if (node.op in ("Relu", "Relu6", "Elu") or \
             (node.op.find("AndRelu") != -1 and \
             ('alpha' not in node.attr or ('alpha' in node.attr and node.attr['alpha'].f == 0)))) \
@@ -198,7 +195,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
                      or len(self.node_name_mapping \
                         [Helper.node_name_from_input(node.input[0])].output) > 1):
                         return True
-        elif 'T' in node.attr and node.attr['T'].type in (dtypes.quint8, dtypes.uint8):
+        elif 'T' in node.attr and dtypes.DType(node.attr['T'].type) in (dtypes.quint8, dtypes.uint8):
             return True
         elif (node.op.find("QuantizedConv") != -1
               or node.op.find("QuantizedDepthwiseConv") != -1 or
@@ -206,7 +203,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
               ) and ((node.op.find("Relu") == -1 and node.op.find("Elu") == -1) or \
               ('alpha' in node.attr and node.attr['alpha'].f > 0)):
             return False
-        elif self._check_op_list(node.op):
+        elif self._check_op_list(node.op) or (self.itex_mode and node.op in ('Add', 'AddV2')):
             if node.op == 'ConcatV2':
                 find_relu = False
                 for i in range(0,node.attr['N'].i):
@@ -216,16 +213,16 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
                         input_node = self.node_name_mapping[node.input[i]].node
                     find_relu |= self._find_relu_node(input_node)
                 return find_relu
+            if re.search(r"\w+:\d+", node.input[0]):
+                input_node = self.node_name_mapping[node.input[0].rsplit(':', 1)[0]].node
             else:
-                if re.search(r"\w+:\d+", node.input[0]):
-                    input_node = self.node_name_mapping[node.input[0].rsplit(':', 1)[0]].node
-                else:
-                    input_node = self.node_name_mapping[node.input[0]].node
-                return self._find_relu_node(input_node)
+                input_node = self.node_name_mapping[node.input[0]].node
+            return self._find_relu_node(input_node)
         else:
             return False
 
     def _insert_qdq_pattern_for_common_ops(self, original_node, is_asymmetric):
+        """Insert QDQ patterns for common OPs."""
         namespace_prefix = original_node.name + "_eightbit"
         if original_node.op in ("Conv2DBackpropInput", "Conv3DBackpropInputV2"):
             all_inputs = self.node_name_mapping[original_node.name].node.input[-1:]
@@ -263,9 +260,11 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
 
 
     def _insert_qdq_pattern_for_concatv2(self, original_node, is_asymmetric):
+        """Insert QDQ patterns for each input of ConcatV2."""
         namespace_prefix = original_node.name + "_eightbit"
-        num_input = len(original_node.input)
-        original_inputs = original_node.input[0:num_input - 1]
+        normal_inputs = [i for i in original_node.input if i[0] != '^']
+        num_input = len(normal_inputs)
+        original_inputs = normal_inputs[0:num_input - 1]
         input_idx = 0
         for original_input_name in original_inputs:
             self._insert_qdq_pattern_for_each_input(original_node.name,
@@ -412,11 +411,12 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
     def _insert_qdq_pattern_for_weight_node(self,
                                             computational_node,
                                             weight_node,
-                                            enter_node,
+                                            weight_name,
                                             min_max_values,
                                             per_channel,
                                             weight_bit=7.0,
                                             device='cpu'):
+        """Insert QDQ pattern for weight node."""
         host_op_type = computational_node.op
         base_name = weight_node.name + "_"
         qint8_const_name = base_name + "qint8_const"
@@ -426,7 +426,10 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
         range_coefficent = 127 / (2 ** weight_bit - 1)
         min_value = 0
         max_value = 0
-       
+        insert_reshape = False
+        shape_convert = None
+        shape_revert = None
+
         # The weight node of BatchMatMul may have no value
         if 'value' in weight_node.attr and \
            host_op_type in ("Conv2D", "MatMul", "BatchMatMul", "BatchMatMulV2", "Conv3D", \
@@ -447,7 +450,7 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
                 ranges[ranges < epsilon] = epsilon
                 min_value[np.abs(min_value) < epsilon] = -epsilon
                 max_value[np.abs(max_value) < epsilon] = epsilon
-                qint8_tensor = (np.around(float_tensor *127.0/ranges)).astype(np.int8)
+                # qint8_tensor = (np.around(float_tensor *127.0/ranges)).astype(np.int8)
             else:
                 min_value = np.min(float_tensor)
                 max_value = np.max(float_tensor)
@@ -462,8 +465,8 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
                     else:
                         max_value = min_value / 2.0
                 range_value = np.max(np.abs([min_value, max_value]))
-                qint8_tensor = (np.around(float_tensor * 127.0 / range_value)).astype(np.int8)
-                qint8_tensor = np.clip(qint8_tensor, -127, 127).astype(np.int8)
+                # qint8_tensor = (np.around(float_tensor * 127.0 / range_value)).astype(np.int8)
+                # qint8_tensor = np.clip(qint8_tensor, -127, 127).astype(np.int8)
                 min_value = -range_value
                 max_value = range_value
         elif host_op_type == "DepthwiseConv2dNative":
@@ -482,10 +485,14 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             # When divide by range, qint8_tensor needs to be 3 dim
             # where, 3rd dim should be same dim of ranges
             a, b, c, d = float_tensor.shape
-            qint8_tensor = (np.around(float_tensor.reshape(a, b, c * d) * 127.0 /
-                            ranges)).astype(np.int8)
+            # qint8_tensor = (np.around(float_tensor.reshape(a, b, c * d) * 127.0 /
+            #                ranges)).astype(np.int8)
             # get the shape back to 4 dim
-            qint8_tensor = qint8_tensor.reshape(a, b, c, d)
+            # qint8_tensor = qint8_tensor.reshape(a, b, c, d)
+            if self.itex_mode and d != 1:
+                insert_reshape = True
+                shape_convert = [a, b, c * d]
+                shape_revert = [a, b, c, d]
         else:
             min_value = np.min(min_max_values[computational_node.name+'__min'])
             max_value = np.max(min_max_values[computational_node.name+'__max'])
@@ -495,28 +502,28 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
         max_node = Helper.create_constant_node(max_name, max_value,
                                                             dtypes.float32, device="cpu")
         if "BatchMatMul" in host_op_type and "BatchMatMul" not in weight_node.op:
-            min_node.input.append("^" + weight_node.name)
-            max_node.input.append("^" + weight_node.name)
+            min_node.input.append("^" + weight_name)
+            max_node.input.append("^" + weight_name)
 
-        quant_const_enter_node = None
         min_enter_node = None
         max_enter_node = None
-        if enter_node:
-            quant_const_enter_node = Helper.create_node('Enter', \
-                                           qint8_const_name + '_enter', [weight_node.name])
-            Helper.set_attr_string(quant_const_enter_node,
-                                           'frame_name', enter_node.attr['frame_name'].s)
-            Helper.set_attr_dtype(quant_const_enter_node, 'T', dtypes.float32)
-            Helper.set_attr_bool(quant_const_enter_node, 'is_constant', True)
-            Helper.set_attr_int(quant_const_enter_node, \
-                                         'parallel_iterations', enter_node.attr['parallel_iterations'].i)
+        if insert_reshape:
+            reshape_dims_4to3_name = qint8_const_name + "_reshape_dims_4to3_"
+            reshape_dims_4to3_node = Helper.create_constant_node(
+                reshape_dims_4to3_name, shape_convert, dtypes.int32)
+            reshape_4to3_name = qint8_const_name + "_reshape_4to3_"
+            reshape_4to3_node = Helper.create_node("Reshape", reshape_4to3_name,
+                                                    [weight_node.name, reshape_dims_4to3_name])
+            reshape_4to3_node.attr["T"].CopyFrom(
+                attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
             quant_node = Helper.create_node(
                     "QuantizeV2", qint8_const_name + '_quant',
-                    [quant_const_enter_node.name, min_name, max_name])
+                    [reshape_4to3_name, min_name, max_name])
         else:
             quant_node = Helper.create_node(
-                    "QuantizeV2", qint8_const_name + '_quant',
-                    [weight_node.name, min_name, max_name])
+                "QuantizeV2", qint8_const_name + '_quant',
+                [weight_node.name, min_name, max_name])
+
         dequant_node = Helper.create_node(
             "Dequantize", base_name + '_dequant',
             [quant_node.name, quant_node.name + ':1', quant_node.name + ':2'])
@@ -526,10 +533,10 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
         Helper.set_attr_dtype(dequant_node, "T", dtypes.qint8)
         Helper.set_attr_string(dequant_node, "mode", b"SCALED")
         if per_channel:
-            if host_op_type == 'Conv2D' or host_op_type == 'Conv2DBackpropInput':
+            if host_op_type in ('Conv2D', 'Conv2DBackpropInput'):
                 Helper.set_attr_int(quant_node, 'axis', 3)
                 Helper.set_attr_int(dequant_node, 'axis', 3)
-            elif host_op_type == 'Conv3D' or host_op_type == 'Conv3DBackpropInputV2':
+            elif host_op_type in ('Conv3D', 'Conv3DBackpropInputV2'):
                 Helper.set_attr_int(quant_node, 'axis', 4)
                 Helper.set_attr_int(dequant_node, 'axis', 4)
             elif host_op_type == 'MatMul':
@@ -542,44 +549,79 @@ class GenerateGraphWithQDQPattern(GraphRewriterBase):
             Helper.set_attr_int(quant_node, 'axis', 2)
             Helper.set_attr_int(dequant_node, 'axis', 2)
 
-        if enter_node:
-            min_enter_node = Helper.create_node('Enter', min_name + '_enter', [min_name])
-            Helper.set_attr_string(min_enter_node,
-                                           'frame_name', enter_node.attr['frame_name'].s)
-            Helper.set_attr_dtype(min_enter_node, 'T', dtypes.float32)
-            Helper.set_attr_bool(min_enter_node, 'is_constant', True)
-            Helper.set_attr_int(min_enter_node, 'parallel_iterations', \
-                                             enter_node.attr['parallel_iterations'].i)
-
-            max_enter_node = Helper.create_node('Enter', max_name + '_enter', [max_name])
-            Helper.set_attr_string(max_enter_node,
-                                           'frame_name', enter_node.attr['frame_name'].s)
-            Helper.set_attr_dtype(max_enter_node, 'T', dtypes.float32)
-            Helper.set_attr_bool(max_enter_node, 'is_constant', True)
-            Helper.set_attr_int(max_enter_node, 'parallel_iterations',\
-                                             enter_node.attr['parallel_iterations'].i)
-
-            self.g_weight.add_node(quant_const_enter_node, weight_node.name, [quant_node.name])
-            self.g_weight.add_node(quant_node, quant_const_enter_node.name, [])
-            self.g_weight.add_node(min_node, None, [min_enter_node.name])
-            self.g_weight.add_node(max_node, None, [max_enter_node.name])
-            self.g_weight.add_node(min_enter_node, min_node.name, [quant_node.name])
-            self.g_weight.add_node(max_enter_node, max_node.name, [quant_node.name])
-            self.g_weight.add_node(dequant_node, quant_node.name, [computational_node.name])
-            computational_node.input[1] = dequant_node.name
-        else:
-            self.g_weight.add_node(quant_node, weight_node.name, [])
+        if insert_reshape:
+            reshape_dims_3to4_name = qint8_const_name + "_reshape_dims_3to4_"
+            reshape_dims_3to4_node = Helper.create_constant_node(
+                reshape_dims_3to4_name, shape_revert, dtypes.int32)
+            reshape_3to4_name = qint8_const_name + "_reshape_3to4_"
+            reshape_3to4_node = Helper.create_node("Reshape", reshape_3to4_name,
+                                                    [dequant_node.name, reshape_dims_3to4_name])
+            reshape_3to4_node.attr["T"].CopyFrom(
+                attr_value_pb2.AttrValue(type=dtypes.float32.as_datatype_enum))
+            self.g_weight.add_node(reshape_dims_4to3_node, None, [reshape_4to3_name])
+            self.g_weight.add_node(reshape_dims_3to4_node, None, [reshape_3to4_name])
+            self.g_weight.add_node(reshape_4to3_node, weight_node.name, [quant_node.name])
+            self.g_weight.add_node(quant_node, reshape_4to3_name, [])
             self.g_weight.add_node(min_node, None, [quant_node.name])
             self.g_weight.add_node(max_node, None, [quant_node.name])
-            self.g_weight.add_node(dequant_node, quant_node.name, [computational_node.name])
-            computational_node.input[1] = dequant_node.name
+            self.g_weight.add_node(dequant_node, quant_node.name, [reshape_3to4_name])
+            self.g_weight.add_node(reshape_3to4_node, dequant_node.name, [computational_node.name])
+            computational_node.input[1] = reshape_3to4_node.name
+        else:
+            if computational_node.name in self.g.parent_frame_details and \
+               self.g.parent_frame_details[computational_node.name]:
+                weight_enter_node = Helper.create_node('Enter', \
+                                            weight_node.name + '_enter', [weight_node.name])
+                Helper.set_attr_string(weight_enter_node, 'frame_name',
+                    self.g.parent_frame_details[computational_node.name].attr['frame_name'].s)
+                Helper.set_attr_dtype(weight_enter_node, 'T', dtypes.float32)
+                Helper.set_attr_bool(weight_enter_node, 'is_constant', True)
+                Helper.set_attr_int(weight_enter_node, 'parallel_iterations', \
+                    self.g.parent_frame_details[computational_node.name].attr['parallel_iterations'].i)
+
+                min_enter_node = Helper.create_node('Enter', min_name + '_enter', [min_name])
+                Helper.set_attr_string(min_enter_node, 'frame_name',
+                    self.g.parent_frame_details[computational_node.name].attr['frame_name'].s)
+                Helper.set_attr_dtype(min_enter_node, 'T', dtypes.float32)
+                Helper.set_attr_bool(min_enter_node, 'is_constant', True)
+                Helper.set_attr_int(min_enter_node, 'parallel_iterations', \
+                 self.g.parent_frame_details[computational_node.name].attr['parallel_iterations'].i)
+
+                max_enter_node = Helper.create_node('Enter', max_name + '_enter', [max_name])
+                Helper.set_attr_string(max_enter_node, 'frame_name',
+                    self.g.parent_frame_details[computational_node.name].attr['frame_name'].s)
+                Helper.set_attr_dtype(max_enter_node, 'T', dtypes.float32)
+                Helper.set_attr_bool(max_enter_node, 'is_constant', True)
+                Helper.set_attr_int(max_enter_node, 'parallel_iterations',\
+                    self.g.parent_frame_details[computational_node.name].attr['parallel_iterations'].i)
+
+                self.g_weight.add_node(quant_node, weight_name, [])
+                self.g_weight.add_node(min_node, None, [min_enter_node.name])
+                self.g_weight.add_node(max_node, None, [max_enter_node.name])
+                self.g_weight.add_node(min_enter_node, min_node.name, [quant_node.name])
+                self.g_weight.add_node(max_enter_node, max_node.name, [quant_node.name])
+                self.g_weight.add_node(weight_enter_node, weight_node.name, [quant_node.name])
+                quant_node.input[0] = weight_enter_node.name
+                quant_node.input[1] = min_enter_node.name
+                quant_node.input[2] = max_enter_node.name
+                self.g_weight.add_node(quant_node, weight_enter_node.name, [])
+                self.g_weight.add_node(dequant_node, quant_node.name, [computational_node.name])
+                computational_node.input[1] = dequant_node.name
+            else:
+                self.g_weight.add_node(quant_node, weight_name, [])
+                self.g_weight.add_node(min_node, None, [quant_node.name])
+                self.g_weight.add_node(max_node, None, [quant_node.name])
+                self.g_weight.add_node(dequant_node, quant_node.name, [computational_node.name])
+                computational_node.input[1] = dequant_node.name
 
     def _ignore_insert_qdq_pattern(self, matched_node_name):
-        if matched_node_name in self.fp32_ops or \
-           matched_node_name in self.bf16_ops:
+        """For some cases we don't need to insert QDQ patterns."""
+        if (matched_node_name in self.fp32_ops or matched_node_name in self.bf16_ops) and \
+           ((matched_node_name,) not in self.quantized_nodes):
             return True
 
-        if matched_node_name not in self.op_wise_config and (matched_node_name, ) not in self.quantized_nodes:
+        if matched_node_name not in self.op_wise_config and \
+           (matched_node_name, ) not in self.quantized_nodes:
             return True
 
         #TODO Remove below two lines once the TF enabled the QuantizedMatMul while
