@@ -14,6 +14,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+"""Convert fp32 op to int8 and fuse the pattern."""
 
 from tensorflow.core.framework import graph_pb2
 from tensorflow.python.platform import gfile
@@ -28,22 +29,11 @@ from .quantize_graph_pooling import FuseNodeStartWithPooling
 from .quantize_graph_bn import FuseNodeStartWithFusedBatchNormV3
 
 class QuantizeGraphForIntel(QuantizeGraphBase):
-    """
-
-    """
+    """Quantize the graph."""
 
     def __init__(self, input_graph, input_node_names, output_node_names, op_wise_config, op_wise_sequences, device, \
                  fake_quant=False, new_api=False, performance_only=False, itex_mode=False):
-        """Quantize Graph For Intel Cpu
-
-        Arguments:
-            input_graph {[type]} -- [description]
-            rules {[type]} -- [description]
-            output_node_names {[type]} -- [description]
-
-        Keyword Arguments:
-            debug {bool} -- [description] (default: {False})
-        """
+        """Quantize Graph For Intel Cpu."""
         super().__init__(output_node_names)
         self.op_wise_config = op_wise_config
         # self.perchannel = perchannel
@@ -80,6 +70,7 @@ class QuantizeGraphForIntel(QuantizeGraphBase):
 
     @dump_elapsed_time("Pass Quantization")
     def do_transform(self):
+        """Apply all the transformers to fuse into int8 op."""
         count = 0
         remove_redundant_quant_flag = False
         op_wise_config_name_list = list(self.op_wise_config.keys())
