@@ -20,7 +20,7 @@ from schema import Schema, And, Use, Optional, Or, Hook
 from ..adaptor import FRAMEWORKS
 from ..strategy import STRATEGIES
 from ..objective import OBJECTIVES
-from ..pruners import PRUNERS
+from ..pruner.pruner_legacy import PRUNERS
 from ..utils import logger
 from ..version import __version__
 import re
@@ -77,6 +77,41 @@ class Pruner():
         self.names= names
         self.parameters = parameters
 
+
+@constructor_register
+class PrunerV2:
+    """
+    similiar to torch optimizer's interface
+    """
+
+    def __init__(self,
+                 target_sparsity=None, pruning_type=None, pattern=None, op_names=None,
+                 excluded_op_names=None,
+                 start_step=None, end_step=None, pruning_scope=None, pruning_frequency=None,
+                 min_sparsity_ratio_per_op=None, max_sparsity_ratio_per_op=None,
+                 sparsity_decay_type=None, pruning_op_types=None, reg_type=None,
+                 criterion_reduce_type=None, parameters=None, resume_from_pruned_checkpoint=None):
+        self.pruner_config = DotDict({
+            'target_sparsity': target_sparsity,
+            'pruning_type': pruning_type,
+            'pattern': pattern,
+            'op_names': op_names,
+            'excluded_op_names': excluded_op_names,  ##global only
+            'start_step': start_step,
+            'end_step': end_step,
+            'pruning_scope': pruning_scope,
+            'pruning_frequency': pruning_frequency,
+            'min_sparsity_ratio_per_op': min_sparsity_ratio_per_op,
+            'max_sparsity_ratio_per_op': max_sparsity_ratio_per_op,
+            'sparsity_decay_type': sparsity_decay_type,
+            'pruning_op_types': pruning_op_types,
+            'reg_type': reg_type,
+            'criterion_reduce_type': criterion_reduce_type,
+            'parameters': parameters,
+            'resume_from_pruned_checkpoint': resume_from_pruned_checkpoint
+        })
+
+
 # Schema library has different loading sequence priorities for different
 # value types.
 # To make sure the fields under dataloader.transform field of yaml file
@@ -217,7 +252,7 @@ ops_schema = Schema({
             lambda s: all(i in ['asym', 'sym', 'asym_float'] for i in s)),
         Optional('dtype'): And(
             list,
-            lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16', 'fp16'] for i in s)),
+            lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16'] for i in s)),
         Optional('algorithm'): And(
             list,
             lambda s: all(i in ['minmax'] for i in s)),
@@ -235,7 +270,7 @@ ops_schema = Schema({
             lambda s: all(i in ['asym', 'sym'] for i in s)),
         Optional('dtype'): And(
             list,
-            lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16', 'fp16'] for i in s)),
+            lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16'] for i in s)),
         # compute_dtypeis only for PyTorch framework
         Optional('compute_dtype', default=['uint8']): And(
             list,
@@ -259,13 +294,13 @@ graph_optimization_schema = Schema({
             Optional('dtype', default=None): And(
                 Or(str, list),
                 Use(input_to_list),
-                lambda s: all(i in ['fp32', 'bf16', 'fp16'] for i in s)),
+                lambda s: all(i in ['fp32', 'bf16'] for i in s)),
         },
         Optional('activation', default=None): {
             Optional('dtype', default=None): And(
                 Or(str, list),
                 Use(input_to_list),
-                lambda s: all(i in ['fp32', 'bf16', 'fp16'] for i in s)),
+                lambda s: all(i in ['fp32', 'bf16'] for i in s)),
             }
     }
 })
@@ -275,20 +310,20 @@ mixed_precision_schema = Schema({
     Optional('precisions', default={'precisions': ['fp32']}): And(
         Or(str, list),
         Use(input_to_list),
-        lambda s: all(i in [ 'fp32', 'bf16', 'fp16'] for i in s)),
+        lambda s: all(i in [ 'fp32', 'bf16'] for i in s)),
 
     Optional('op_wise', default={'weight': {}, 'activation': {}}): {
         Optional('weight', default=None): {
             Optional('dtype', default=None): And(
                 Or(str, list),
                 Use(input_to_list),
-                lambda s: all(i in ['fp32', 'bf16', 'fp16'] for i in s)),
+                lambda s: all(i in ['fp32', 'bf16'] for i in s)),
         },
         Optional('activation', default=None): {
             Optional('dtype', default=None): And(
                 Or(str, list),
                 Use(input_to_list),
-                lambda s: all(i in ['fp32', 'bf16', 'fp16'] for i in s)),
+                lambda s: all(i in ['fp32', 'bf16'] for i in s)),
             }
     }
 })
@@ -646,18 +681,28 @@ criterion_schema = Schema({
     },
     Optional('IntermediateLayersKnowledgeDistillationLoss'): {
         'layer_mappings':
-            And(Or(tuple, list), lambda s: all(len(i) in [2, 4] for i in s)),
+            And(Or(tuple, list), lambda s: all(len(i) in [1, 2] for i in s)),
         Optional('loss_types'):
             And(Or(tuple, list), lambda s: all(i in ['MSE', 'KL', 'L1'] for i in s)),
         Optional('loss_weights'):
             And(Or(tuple, list), lambda s: all(i >= 0 for i in s)),
         Optional('add_origin_loss'): bool,
+    },
+    Optional('SelfKnowledgeDistillationLoss'): {
+        'layer_mappings':
+            And(Or(tuple, list), lambda s: all(len(i) >= 1 for i in s)),
+        Optional('loss_types'):
+            And(Or(tuple, list), lambda s: all(i in ['L2', 'CE', 'KL'] for i in s)),
+        Optional('loss_weights'):
+            And(Or(tuple, list), lambda s: all(i >= 0.0 and i < 1.0 for i in s)),
+        Optional('add_origin_loss'): bool,
+        Optional('temperature'): And(float, lambda s: s > 0),
     }
 })
 
 train_schema = Schema({
-    'optimizer': optimizer_schema,
     'criterion': criterion_schema,
+    Optional('optimizer', default={'SGD': {'learning_rate': 0.001}}): optimizer_schema,
     Optional('dataloader'): dataloader_schema,
     Optional('epoch', default=1): int,
     Optional('start_epoch', default=0): int,
@@ -695,12 +740,36 @@ weight_compression_schema = Schema({
                                lambda s: all(isinstance(i, Pruner) for i in s))
 })
 
+weight_compression_v2_schema = Schema({
+    Optional('target_sparsity', default=0.9): float,
+    Optional('pruning_type', default="snip_momentum"): str,
+    Optional('pattern', default="4x1"): str,
+    Optional('op_names', default=[]): list,
+    Optional('excluded_op_names', default=[]): list,
+    Optional('start_step', default=0): int,
+    Optional('end_step', default=0): int,
+    Optional('pruning_scope', default="global"): str,
+    Optional('pruning_frequency', default=1): int,
+    Optional('min_sparsity_ratio_per_op', default=0.0): float,
+    Optional('max_sparsity_ratio_per_op', default=0.98): float,
+    Optional('sparsity_decay_type', default="exp"): str,
+    Optional('pruning_op_types', default=['Conv', 'Linear']): list,
+    Optional('reg_type', default=None): str,
+    Optional('criterion_reduce_type', default="mean"): str,
+    Optional('parameters', default={"reg_coeff": 0.0}): dict,
+    Optional('resume_from_pruned_checkpoint', default=False): str,
+    Optional('pruners'): And(list, \
+                             lambda s: all(isinstance(i, Pruner) for i in s))
+})
+
+
 # weight_compression_pytorch_schema = Schema({},ignore_extra_keys=True)
 
 approach_schema = Schema({
     Hook('weight_compression', handler=_valid_prune_sparsity): object,
     Hook('weight_compression_pytorch', handler=_valid_prune_sparsity): object,
     Optional('weight_compression'): weight_compression_schema,
+    Optional('weight_compression_v2'): weight_compression_v2_schema,
     Optional('weight_compression_pytorch'): weight_compression_schema,
 })
 
@@ -720,6 +789,7 @@ schema = Schema({
         'framework': And(str, lambda s: s in list(FRAMEWORKS.keys()) + ['NA']),
         Optional('inputs', default=[]): And(Or(str, list), Use(input_to_list)),
         Optional('outputs', default=[]): And(Or(str, list), Use(input_to_list)),
+ 
     },
     Optional('version', default=float(__version__.split('.')[0])): And(
                                           Or(float,
@@ -731,9 +801,12 @@ schema = Schema({
                                       'calibration': {'sampling_size': [100]}, \
                                       'recipes': {'scale_propagation_max_pooling': True,
                                                       'scale_propagation_concat': True,
-                                                      'first_conv_or_matmul_quantization': True},
+                                                      'first_conv_or_matmul_quantization': True,
+                                                      'last_conv_or_matmul_quantization': True,
+                                                      'pre_post_process_quantization': True},
                                       'model_wise': {'weight': {'bit': [7.0]},
                                                      'activation': {}},
+                                      'quant_level': 1,
                                       }): {
         Optional('approach', default='post_training_static_quant'): And(
             str,
@@ -753,13 +826,19 @@ schema = Schema({
         },
         Optional('recipes', default={'scale_propagation_max_pooling': True,
                                          'scale_propagation_concat': True,
-                                         'first_conv_or_matmul_quantization': True}): {
+                                         'first_conv_or_matmul_quantization': True,
+                                         'last_conv_or_matmul_quantization': True,
+                                         'pre_post_process_quantization': True}): {
             Optional('scale_propagation_max_pooling', default=True):
                     And(bool, lambda s: s in [True, False]),
             Optional('scale_propagation_concat', default=True):
                     And(bool, lambda s: s in [True, False]),
             Optional('first_conv_or_matmul_quantization', default=True):
                     And(bool, lambda s: s in [True, False]),
+            Optional('last_conv_or_matmul_quantization', default=True):
+                    And(bool, lambda s: s in [True, False]),
+            Optional('pre_post_process_quantization', default=True):
+                    And(bool, lambda s: s in [True, False]),
             Optional('fast_bias_correction', default=False):
                     And(bool, lambda s: s in [True, False]),
             Optional('weight_correction', default=False):
@@ -779,7 +858,7 @@ schema = Schema({
                 Optional('dtype', default=None): And(
                     Or(str, list),
                     Use(input_to_list),
-                    lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16', 'fp16'] for i in s)),
+                    lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16'] for i in s)),
                 Optional('algorithm', default=None): And(
                     Or(str, list),
                     Use(input_to_list),
@@ -802,7 +881,7 @@ schema = Schema({
                 Optional('dtype', default=None): And(
                     Or(str, list),
                     Use(input_to_list),
-                    lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16', 'fp16'] for i in s)),
+                    lambda s: all(i in ['int8', 'uint8', 'fp32', 'bf16'] for i in s)),
                 # compute_dtypeis only for PyTorch framework
                 Optional('compute_dtype', default=['uint8']): And(
                     Or(str, list),
@@ -821,15 +900,17 @@ schema = Schema({
         Optional('op_wise', default=None): {
             str: ops_schema
         },
+        Optional('quant_level', default=1): And(int, lambda level: level in [0, 1]),
     },
-
+    Optional('use_bf16', default=True): bool,
+    Optional('quant_level', default=1): And(int, lambda level: level in [0, 1]),
     Optional('graph_optimization'): graph_optimization_schema,
     Optional('mixed_precision'): mixed_precision_schema,
 
     Optional('model_conversion'): model_conversion_schema,
 
     Optional('tuning', default={
-        'strategy': {'name': 'basic'},
+        'strategy': {'name': 'basic'}, 
         'accuracy_criterion': {'relative': 0.01, 'higher_is_better': True},
         'objective': 'performance',
         'exit_policy': {'timeout': 0, 'max_trials': 100, 'performance_only': False},
@@ -838,11 +919,14 @@ schema = Schema({
         'diagnosis': False,
         }): {
         Optional('strategy', default={'name': 'basic'}): {
-            'name': And(str, lambda s: s in STRATEGIES), Optional('sigopt_api_token'): str,
+            'name': And(str, lambda s: s in STRATEGIES), 
+            Optional('sigopt_api_token'): str,
             Optional('sigopt_project_id'): str,
             Optional('sigopt_experiment_name', default='nc-tune'): str,
             Optional('accuracy_weight', default=1.0): float,
-            Optional('latency_weight', default=1.0): float
+            Optional('latency_weight', default=1.0): float,
+            Optional('confidence_batches', default=2): int,
+            Optional('hawq_v2_loss', default=None): object,
         } ,
         Hook('accuracy_criterion', handler=_valid_accuracy_field): object,
         Optional('accuracy_criterion', default={'relative': 0.01}): {
@@ -1066,7 +1150,9 @@ schema = Schema({
             Optional("num_evals", default=100000): int,
             Optional("results_csv_path", default=None): str,
             Optional("dataset_path", default=None): str,
+            Optional("supernet_ckpt_path", default=None): str,
             Optional("batch_size", default=64): int,
+            Optional("num_workers", default=20): int,
             },
     },
 
@@ -1086,11 +1172,14 @@ quantization_default_schema = Schema({
                                       'calibration': {'sampling_size': [100]},
                                       'recipes': {'scale_propagation_max_pooling': True,
                                                       'scale_propagation_concat': True,
-                                                      'first_conv_or_matmul_quantization': True},
+                                                      'first_conv_or_matmul_quantization': True,
+                                                      'last_conv_or_matmul_quantization': True,
+                                                      'pre_post_process_quantization': True},
                                       'model_wise': {'weight': {'bit': [7.0]},
                                                      'activation': {}},
                                     }): dict,
-
+    Optional('use_bf16', default=False): bool,
+    Optional('quant_level', default=1): int,
     Optional('tuning', default={
         'strategy': {'name': 'basic'},
         'accuracy_criterion': {'relative': 0.01, 'higher_is_better': True},
@@ -1111,6 +1200,8 @@ pruning_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'random_seed': 1978, 'tensorboard': False,
         'workspace': {'path': default_workspace}}): dict,
@@ -1135,10 +1226,14 @@ graph_optimization_default_schema = Schema({
                                     'calibration': {'sampling_size': [100]},
                                     'recipes': {'scale_propagation_max_pooling': True,
                                                     'scale_propagation_concat': True,
-                                                    'first_conv_or_matmul_quantization': True},
+                                                    'first_conv_or_matmul_quantization': True,
+                                                    'last_conv_or_matmul_quantization': True,
+                                                    'pre_post_process_quantization': True},
                                     'model_wise': {'weight': {'bit': [7.0]},
                                                     'activation': {}}}): dict,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'strategy': {'name': 'basic'},
         'accuracy_criterion': {'relative': 0.01, 'higher_is_better': True},
@@ -1165,10 +1260,14 @@ mixed_precision_default_schema = Schema({
                                     'calibration': {'sampling_size': [100]},
                                     'recipes': {'scale_propagation_max_pooling': True,
                                                     'scale_propagation_concat': True,
-                                                    'first_conv_or_matmul_quantization': True},
+                                                    'first_conv_or_matmul_quantization': True,
+                                                    'last_conv_or_matmul_quantization': True,
+                                                    'pre_post_process_quantization': True},
                                     'model_wise': {'weight': {'bit': [7.0]},
                                                     'activation': {}}}): dict,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'strategy': {'name': 'basic'},
         'accuracy_criterion': {'relative': 0.01, 'higher_is_better': True},
@@ -1191,11 +1290,15 @@ benchmark_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('quantization', default={'approach': 'post_training_static_quant', 
                                     'calibration': {'sampling_size': [100]},
                                     'recipes': {'scale_propagation_max_pooling': True,
                                                     'scale_propagation_concat': True,
-                                                    'first_conv_or_matmul_quantization': True},
+                                                    'first_conv_or_matmul_quantization': True,
+                                                    'last_conv_or_matmul_quantization': True,
+                                                    'pre_post_process_quantization': True},
                                     'model_wise': {'weight': {'bit': [7.0]},
                                                     'activation': {}}}): dict,
 
@@ -1219,6 +1322,8 @@ distillation_default_schema = Schema({
 
     Optional('device', default='cpu'): str,
 
+    Optional('use_bf16', default=False): bool,
+
     Optional('tuning', default={
         'random_seed': 1978, 'tensorboard': False,
         'workspace': {'path': default_workspace}}): dict,
@@ -1286,57 +1391,96 @@ class Conf(object):
             )
 
     def map_pyconfig_to_cfg(self, pythonic_config):
-        mapping = {
-            'device': pythonic_config.quantization.device,
-            'model.inputs': pythonic_config.quantization.inputs,
-            'model.outputs': pythonic_config.quantization.outputs,
-            'model.framework': pythonic_config.quantization.backend,
-            'quantization.approach': pythonic_config.quantization.approach,
-            'quantization.calibration.sampling_size': 
-                pythonic_config.quantization.calibration_sampling_size,
-            'quantization.optype_wise': pythonic_config.quantization.op_type_list,
-            'quantization.op_wise': pythonic_config.quantization.op_type_list,
-            'distillation.train.criterion': pythonic_config.distillation.criterion,
-            'distillation.train.optimizer': pythonic_config.distillation.optimizer,
-            'pruning.approach.weight_compression': pythonic_config.pruning.weight_compression,
-            'nas.approach': pythonic_config.nas.approach,
-            'nas.search': pythonic_config.nas.search,
-            'nas.dynas': pythonic_config.nas.dynas,
-            'tuning.strategy.name': pythonic_config.quantization.strategy,
-            'tuning.accuracy_criterion.relative': 
-                pythonic_config.quantization.accuracy_criterion.relative,
-            'tuning.accuracy_criterion.absolute':
-                pythonic_config.quantization.accuracy_criterion.absolute,
-            'tuning.accuracy_criterion.higher_is_better':
-                pythonic_config.quantization.accuracy_criterion.higher_is_better,
-            'tuning.objective': pythonic_config.quantization.objective,
-            'tuning.exit_policy.timeout': pythonic_config.quantization.timeout,
-            'tuning.exit_policy.max_trials': pythonic_config.quantization.max_trials,
-            'tuning.exit_policy.performance_only': pythonic_config.quantization.performance_only,
-            'tuning.random_seed': pythonic_config.options.random_seed,
-            'tuning.workspace.path': pythonic_config.options.workspace,
-            'tuning.workspace.resume': pythonic_config.options.resume_from,
-            'evaluation.performance.warmup': pythonic_config.benchmark.warmup,
-            'evaluation.performance.iteration': pythonic_config.benchmark.iteration,
-            'evaluation.performance.configs.cores_per_instance':
-                pythonic_config.benchmark.cores_per_instance,
-            'evaluation.performance.configs.num_of_instance':
-                pythonic_config.benchmark.num_of_instance,
-            'evaluation.performance.configs.inter_num_of_threads':
-                pythonic_config.benchmark.inter_num_of_threads,
-            'evaluation.performance.configs.intra_num_of_threads':
-                pythonic_config.benchmark.intra_num_of_threads,
-            'evaluation.accuracy.configs.cores_per_instance':
-                pythonic_config.benchmark.cores_per_instance,
-            'evaluation.accuracy.configs.num_of_instance':
-                pythonic_config.benchmark.num_of_instance,
-            'evaluation.accuracy.configs.inter_num_of_threads':
-                pythonic_config.benchmark.inter_num_of_threads,
-            'evaluation.accuracy.configs.intra_num_of_threads':
-                pythonic_config.benchmark.intra_num_of_threads,
-            'use_bf16': pythonic_config.quantization.use_bf16,
-            'reduce_range': pythonic_config.quantization.reduce_range
-        }
+        mapping = {}
+        if pythonic_config.quantization is not None:
+            mapping.update({
+                'device': pythonic_config.quantization.device,
+                'model.inputs': pythonic_config.quantization.inputs,
+                'model.outputs': pythonic_config.quantization.outputs,
+                'model.backend': pythonic_config.quantization.backend,
+                'model.quant_format': pythonic_config.quantization.quant_format,
+                'quantization.approach': pythonic_config.quantization.approach,
+                'quantization.calibration.sampling_size': 
+                    pythonic_config.quantization.calibration_sampling_size,
+                'quantization.optype_wise': pythonic_config.quantization.op_type_list,
+                'quantization.op_wise': pythonic_config.quantization.op_name_list,
+                'tuning.strategy.name': pythonic_config.quantization.strategy,
+                'tuning.accuracy_criterion.relative': 
+                    pythonic_config.quantization.accuracy_criterion.relative,
+                'tuning.accuracy_criterion.absolute':
+                    pythonic_config.quantization.accuracy_criterion.absolute,
+                'tuning.accuracy_criterion.higher_is_better':
+                    pythonic_config.quantization.accuracy_criterion.higher_is_better,
+                'tuning.objective': pythonic_config.quantization.objective,
+                'tuning.exit_policy.timeout': pythonic_config.quantization.timeout,
+                'tuning.exit_policy.max_trials': pythonic_config.quantization.max_trials,
+                'tuning.exit_policy.performance_only': pythonic_config.quantization.performance_only,
+                'use_bf16': pythonic_config.quantization.use_bf16,
+                'quantization.quant_level': pythonic_config.quantization.quant_level,
+                'reduce_range': pythonic_config.quantization.reduce_range
+            })
+            if pythonic_config.quantization.strategy_kwargs:
+                st_kwargs = pythonic_config.quantization.strategy_kwargs
+                for st_key in ['sigopt_api_token', 'sigopt_project_id', 'sigopt_experiment_name', \
+                    'accuracy_weight', 'latency_weight', 'hawq_v2_loss']:
+
+                    if st_key in st_kwargs:
+                        st_val =  st_kwargs[st_key]
+                        mapping.update({'tuning.strategy.' + st_key: st_val})
+            
+        if pythonic_config.distillation is not None:
+            mapping.update({
+                'distillation.train.criterion': pythonic_config.distillation.criterion,
+                'distillation.train.optimizer': pythonic_config.distillation.optimizer,
+            })
+        if pythonic_config.pruning is not None:
+            mapping.update({
+                'pruning.approach.weight_compression': pythonic_config.pruning.weight_compression,
+            })
+        if pythonic_config.nas is not None:
+            mapping.update({
+                'nas.approach': pythonic_config.nas.approach,
+                'nas.search': pythonic_config.nas.search,
+                'nas.dynas': pythonic_config.nas.dynas,
+            })
+        if pythonic_config.options is not None:
+            mapping.update({
+                'tuning.random_seed': pythonic_config.options.random_seed,
+                'tuning.workspace.path': pythonic_config.options.workspace,
+                'tuning.workspace.resume': pythonic_config.options.resume_from,
+                'tuning.tensorboard': pythonic_config.options.tensorboard,
+            })
+        if pythonic_config.benchmark is not None:
+            if pythonic_config.benchmark.inputs != []:
+                mapping.update({'model.inputs': pythonic_config.benchmark.inputs})
+            if pythonic_config.benchmark.outputs != []:
+                mapping.update({'model.outputs': pythonic_config.benchmark.outputs})
+            mapping.update({
+                'model.backend': pythonic_config.benchmark.backend,
+                'evaluation.performance.warmup': pythonic_config.benchmark.warmup,
+                'evaluation.performance.iteration': pythonic_config.benchmark.iteration,
+                'evaluation.performance.configs.cores_per_instance':
+                    pythonic_config.benchmark.cores_per_instance,
+                'evaluation.performance.configs.num_of_instance':
+                    pythonic_config.benchmark.num_of_instance,
+                'evaluation.performance.configs.inter_num_of_threads':
+                    pythonic_config.benchmark.inter_num_of_threads,
+                'evaluation.performance.configs.intra_num_of_threads':
+                    pythonic_config.benchmark.intra_num_of_threads,
+                'evaluation.accuracy.configs.cores_per_instance':
+                    pythonic_config.benchmark.cores_per_instance,
+                'evaluation.accuracy.configs.num_of_instance':
+                    pythonic_config.benchmark.num_of_instance,
+                'evaluation.accuracy.configs.inter_num_of_threads':
+                    pythonic_config.benchmark.inter_num_of_threads,
+                'evaluation.accuracy.configs.intra_num_of_threads':
+                    pythonic_config.benchmark.intra_num_of_threads,
+            })
+
+        if "model.backend" not in mapping:
+            mapping.update({
+                'model.backend': "default",
+            })
 
         for k, v in mapping.items():
             if k in ['tuning.accuracy_criterion.relative', 'tuning.accuracy_criterion.absolute']:
