@@ -15,19 +15,20 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+"""Fefine the tuning strategy that uses tpe search in tuning space."""
 import copy
 import os
 from pathlib import Path
 from functools import partial
 import numpy as np
-import hyperopt as hpo
-from hyperopt import fmin, hp, STATUS_OK, Trials
 from neural_compressor.utils import logger
+from neural_compressor.utils.utility import LazyImport
 from neural_compressor.strategy.strategy import strategy_registry, TuneStrategy
 from collections import OrderedDict
-from neural_compressor.strategy.st_utils.tuning_sampler import OpWiseTuningSampler
-from neural_compressor.strategy.st_utils.tuning_structs import OpTuningConfig
+from neural_compressor.strategy.utils.tuning_sampler import OpWiseTuningSampler
+from neural_compressor.strategy.utils.tuning_structs import OpTuningConfig
 
+hyperopt = LazyImport('hyperopt')
 
 try:
     import pandas as pd
@@ -83,12 +84,28 @@ class TpeTuneStrategy(TuneStrategy):
     """
     def __init__(self, model, conf, q_dataloader, q_func=None,
                  eval_dataloader=None, eval_func=None, dicts=None, q_hooks=None):
+        """Initialize the tpe tuning strategy if the user specified to use it."""
         assert conf.usr_cfg.quantization.approach == 'post_training_static_quant', \
                "TPE strategy is only for post training static quantization!"
+        """Initialize the tpe tuning strategy if the user specified to use it."""
+        strategy_name = conf.usr_cfg.tuning.strategy.name
+        if strategy_name.lower() == "tpe":
+            try:
+                import hyperopt
+            except ImportError:
+                try:
+                    import subprocess
+                    import sys
+                    subprocess.check_call([sys.executable, "-m", "pip", "install", "hyperopt"])
+                    import hyperopt # pylint: disable=import-error
+                except:
+                    assert False, "Unable to import hyperopt from the local environment."
+        else:
+            pass
         self.hpopt_search_space = None
         self.warm_start = False
         self.cfg_evaluated = False
-        self.hpopt_trials = Trials()
+        self.hpopt_trials = hyperopt.Trials()
         self.max_trials = conf.usr_cfg.tuning.exit_policy.get('max_trials', 200)
         self.loss_function_config = {
             'acc_th': conf.usr_cfg.tuning.accuracy_criterion.relative if \
@@ -138,9 +155,10 @@ class TpeTuneStrategy(TuneStrategy):
         return save_dict
 
     def _configure_hpopt_search_space_and_params(self, search_space):
+        """Set the configuration of hpopt searching strategy."""
         self.hpopt_search_space = {}
         for param, configs in search_space.items():
-            self.hpopt_search_space[(param)] = hp.choice((param[0]), configs)
+            self.hpopt_search_space[(param)] = hyperopt.hp.choice((param[0]), configs)
         # Find minimum number of choices for params with more than one choice
         multichoice_params = [len(configs) for param, configs in search_space.items()
                               if len(configs) > 1]
@@ -149,7 +167,7 @@ class TpeTuneStrategy(TuneStrategy):
         min_param_size = min(multichoice_params) if len(multichoice_params) > 0 else 1
         self.tpe_params['n_EI_candidates'] = min_param_size
         self.tpe_params['prior_weight'] = 1 / min_param_size
-        self._algo = partial(hpo.tpe.suggest,
+        self._algo = partial(hyperopt.tpe.suggest,
                             n_startup_jobs=self.tpe_params['n_initial_point'],
                             gamma=self.tpe_params['gamma'],
                             n_EI_candidates=self.tpe_params['n_EI_candidates'],
@@ -157,9 +175,7 @@ class TpeTuneStrategy(TuneStrategy):
         return True
 
     def traverse(self):
-        """Tpe traverse logic.
-
-        """
+        """Tpe traverse logic."""
         logger.info("Start to run tpe strategy.")
         # prepare log file
         trials_file = os.path.join(os.path.dirname(self.history_path), 'tpe_trials.csv')
@@ -181,7 +197,7 @@ class TpeTuneStrategy(TuneStrategy):
         calib_sampling_size_lst = tuning_space.root_item.get_option_by_name('calib_sampling_size').options
         # step1. collect the ops that support static and dynamic
         quant_mode_wise_items = OrderedDict()
-        query_order = ['static', 'dynamic', 'bf16', 'fp16', 'fp32']
+        query_order = ['static', 'dynamic', 'bf16', 'fp32']
         pre_items = set()
         for quant_mode in query_order:
             items = tuning_space.query_items_by_quant_mode(quant_mode)
@@ -225,12 +241,12 @@ class TpeTuneStrategy(TuneStrategy):
             self._configure_hpopt_search_space_and_params(first_run_cfg)
             # Run first iteration with best result from history
             trials_count = len(self.hpopt_trials.trials) + 1
-            fmin(partial(self.object_evaluation, model=self.model),
-                space=self.hpopt_search_space,
-                algo=self._algo,
-                max_evals=trials_count,
-                trials=self.hpopt_trials,
-                show_progressbar=False)
+            hyperopt.fmin(partial(self.object_evaluation, model=self.model),
+                          space=self.hpopt_search_space,
+                          algo=self._algo,
+                          max_evals=trials_count,
+                          trials=self.hpopt_trials,
+                          show_progressbar=False)
             if pd is not None:
                 self._save_trials(trials_file)
                 self._update_best_result(best_result_file)
@@ -266,12 +282,12 @@ class TpeTuneStrategy(TuneStrategy):
                 self.cfg_evaluated = False
                 logger.debug("Trial iteration start: {} / {}.".format(
                     trials_count, self.max_trials))
-                fmin(partial(self.object_evaluation, model=self.model),
-                    space=self.hpopt_search_space,
-                    algo=self._algo,
-                    max_evals=trials_count,
-                    trials=self.hpopt_trials,
-                    show_progressbar=False)
+                hyperopt.fmin(partial(self.object_evaluation, model=self.model),
+                              space=self.hpopt_search_space,
+                              algo=self._algo,
+                              max_evals=trials_count,
+                              trials=self.hpopt_trials,
+                              show_progressbar=False)
                 trials_count += 1
                 if pd is not None:
                     self._save_trials(trials_file)
@@ -283,6 +299,7 @@ class TpeTuneStrategy(TuneStrategy):
             logger.warn("Can't create search space for input model.")
 
     def _prepare_final_searchspace(self, first, second):
+        """Set the final search space."""
         for key, cfgs in second.items():
             new_cfg = []
             for cfg in cfgs:
@@ -292,6 +309,7 @@ class TpeTuneStrategy(TuneStrategy):
         return first
 
     def add_loss_to_tuned_history_and_find_best(self, tuning_history_list):
+        """Find the best tuned history."""
         logger.debug("Number of resumed configs is {}.".format(len(tuning_history_list)))
         best_loss = None
         first_run_cfg = None
@@ -316,7 +334,7 @@ class TpeTuneStrategy(TuneStrategy):
         return first_run_cfg
 
     def object_evaluation(self, tune_cfg, model):
-        # check if config was alredy evaluated
+        """Check if config was alredy evaluated."""
         op_cfgs = self._tune_cfg_converter(tune_cfg)
         self.last_qmodel = self.adaptor.quantize(op_cfgs, self.model, self.calib_dataloader)
         self.last_tune_result = self._evaluate(self.last_qmodel)
@@ -349,7 +367,7 @@ class TpeTuneStrategy(TuneStrategy):
             'acc_loss': acc_diff,
             'lat_diff': lat_diff,
             'quantization_ratio': quantization_ratio,
-            'status': STATUS_OK}
+            'status': hyperopt.STATUS_OK}
 
     def _calculate_acc_lat_diff(self, acc, lat):
         int8_acc = acc
@@ -361,6 +379,7 @@ class TpeTuneStrategy(TuneStrategy):
         return acc_diff, lat_diff
 
     def calculate_loss(self, acc_diff, lat_diff, config):
+        """Calculate the accuracy loss."""
         gamma_penalty = 40  # penalty term
         acc_loss_component = self._calculate_acc_loss_component(acc_diff)
         lat_loss_component = self._calculate_lat_diff_component(lat_diff)
@@ -396,7 +415,7 @@ class TpeTuneStrategy(TuneStrategy):
         config['lat_scale'] = 10 / np.abs(lat_max - lat_min)
 
     def _save_trials(self, trials_log):
-        """ save trial result to log file"""
+        """Save the trial result to the log file."""
         tpe_trials_results = pd.DataFrame(self.hpopt_trials.results)
         csv_file = trials_log
         tpe_trials_results.to_csv(csv_file)
@@ -449,11 +468,10 @@ class TpeTuneStrategy(TuneStrategy):
                                                         self.best_result['quantization_ratio']))
 
     def stop(self, timeout, trials_count):
-        """Check if need to stop traversing the tuning space, either accuracy goal is met
-           or timeout is reach.
+        """Check if need to stop traversing the tuning space, either accuracy goal is met or timeout is reach.
 
         Returns:
-            bool: True if need stop, otherwise False
+            bool: True if need stop, otherwise False.
         """
         need_stop = False
         if not self.cfg_evaluated:
