@@ -1,3 +1,5 @@
+"""Common methods for DyNAS."""
+
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
@@ -25,16 +27,23 @@ from typing import Tuple
 import numpy as np
 import ofa
 from fvcore.nn import FlopCountAnalysis
-from ofa.imagenet_classification.data_providers.imagenet import ImagenetDataProvider
-from ofa.imagenet_classification.run_manager import ImagenetRunConfig, RunManager
-from ofa.tutorial.flops_table import rm_bn_from_net
-
-from neural_compressor.experimental.nas.dynast.dynas_manager import ParameterManager
+from neural_compressor.experimental.nas.dynast.dynas_manager import \
+    ParameterManager
 from neural_compressor.experimental.nas.dynast.dynas_predictor import Predictor
+# from neural_compressor.experimental.nas.dynast.supernetwork.machine_translation.transformer_interface import (
+#     compute_bleu, compute_latency, compute_macs)
 from neural_compressor.utils.utility import LazyImport, logger
+from ofa.imagenet_classification.data_providers.imagenet import \
+    ImagenetDataProvider
+from ofa.imagenet_classification.run_manager import (ImagenetRunConfig,
+                                                     RunManager)
+from ofa.tutorial.flops_table import rm_bn_from_net
 
 torch = LazyImport('torch')
 torchvision = LazyImport('torchvision')
+transformer_interface = LazyImport(
+    'neural_compressor.experimental.nas.dynast.supernetwork.machine_translation.transformer_interface'
+)
 
 
 def get_macs(
@@ -42,6 +51,16 @@ def get_macs(
     input_size: Tuple[int, int, int, int] = (1, 3, 224, 224),
     device: str = 'cpu',
 ) -> int:
+    """Get the MACs of the model.
+
+    Args:
+        model (torch.nn.Module): The torch model.
+        input_size (Tuple): The input dimension.
+        device (str): The device on which the model runs.
+
+    Returns:
+        The MACs of the model.
+    """
     model = model.to(device)
     rm_bn_from_net(model)
     model.eval()
@@ -57,7 +76,7 @@ def _auto_steps(
     min_steps: int = 25,
     min_samples: int = 500,
 ) -> int:
-    """Simple scaling of number of steps w.r.t batch_size
+    """Scaling of number of steps w.r.t batch_size.
 
     Example:
     1. `_auto_steps(1, True), _auto_steps(1, False)` -> 100, 500
@@ -66,16 +85,15 @@ def _auto_steps(
     4. `_auto_steps(32, True), _auto_steps(8, False)` -> 5, 25
 
     Args:
-    - batch_size
-    - is_warmup: if set to True, will scale down the number of steps by `warmup_scale`.
-    - warmup_scale: scale by which number of steps should be decreased if `is_warmup` is True.
-    - min_steps: minimum number of steps to return
-    - min_samples: returned steps multiplied by `batch_size` should be at least this much.
+        batch_size (int): size of a batch input data.
+        is_warmup (bool): if set to True, will scale down the number of steps by `warmup_scale`.
+        warmup_scale (float): scale by which number of steps should be decreased if `is_warmup` is True.
+        min_steps (int): minimum number of steps to return.
+        min_samples (int): returned steps multiplied by `batch_size` should be at least this much.
 
     Returns:
-        number of steps
+        number of steps.
     """
-
     if not is_warmup:
         warmup_scale = 1.0
 
@@ -93,16 +111,16 @@ def measure_latency(
     """Measure Torch model's latency.
 
     Args:
-    - model: Torch model
-    - input_size: a tuple (batch size, channels, resolution, resolution).
-    - warmup_steps - how many data batches to use to warm up the device.
-        If 'None' it will be adjusted automatically w.r.t batch size.
-    - measure_steps - how many data batches to use for latency measurement.
-        If 'None' it will be adjusted automatically w.r.t batch size.
-    - device: which device is being used for latency measurement.
+        model (torch.nn.Module): Torch model.
+        input_size (Tuple): a tuple (batch size, channels, resolution, resolution).
+        warmup_steps (int): how many data batches to use to warm up the device.
+          If 'None' it will be adjusted automatically w.r.t batch size.
+        measure_steps (int): how many data batches to use for latency measurement.
+          If 'None' it will be adjusted automatically w.r.t batch size.
+        device (str): which device is being used for latency measurement.
 
     Returns:
-        mean latency; std latency
+        mean latency; std latency.
     """
     if not warmup_steps:
         warmup_steps = _auto_steps(input_size[0], is_warmup=True)
@@ -141,11 +159,15 @@ def measure_latency(
 
 
 class Runner:
+    """The Runner base class."""
+
     pass
 
 
 class OFARunner(Runner):
-    """The OFARunner class manages the sub-network selection from the OFA super-network and
+    """The OFARunner class.
+
+    The OFARunner class manages the sub-network selection from the OFA super-network and
     the validation measurements of the sub-networks. ResNet50, MobileNetV3 w1.0, and MobileNetV3 w1.2
     are currently supported. Imagenet is required for these super-networks `imagenet-ilsvrc2012`.
     """
@@ -156,24 +178,35 @@ class OFARunner(Runner):
         acc_predictor: Predictor,
         macs_predictor: Predictor,
         latency_predictor: Predictor,
-        imagenetpath: str,
+        datasetpath: str,
         batch_size: int,
+        num_workers: int = 20,
+        **kwargs,
     ) -> None:
+        """Initialize the attributes."""
         self.supernet = supernet
         self.acc_predictor = acc_predictor
         self.macs_predictor = macs_predictor
         self.latency_predictor = latency_predictor
         self.device = 'cpu'
         self.test_size = None
-        ImagenetDataProvider.DEFAULT_PATH = imagenetpath
+        ImagenetDataProvider.DEFAULT_PATH = datasetpath
         self.ofa_network = ofa.model_zoo.ofa_net(supernet, pretrained=True)
-        self.run_config = ImagenetRunConfig(test_batch_size=64, n_worker=20)
+        self.run_config = ImagenetRunConfig(test_batch_size=64, n_worker=num_workers)
         self.batch_size = batch_size
 
     def estimate_accuracy_top1(
         self,
         subnet_cfg: dict,
     ) -> float:
+        """Estimate the top1 accuracy of the subnet with the predictor.
+
+        Args:
+            subnet_cfg (dict): The dictionary describing the subnet.
+
+        Returns:
+            Top1 accuracy of the subnet.
+        """
         top1 = self.acc_predictor.predict(subnet_cfg)
         return top1
 
@@ -181,6 +214,14 @@ class OFARunner(Runner):
         self,
         subnet_cfg: dict,
     ) -> int:
+        """Estimate the MACs of the subnet with the predictor.
+
+        Args:
+            subnet_cfg (dict): The dictionary describing the subnet.
+
+        Returns:
+            MACs of the subnet.
+        """
         macs = self.macs_predictor.predict(subnet_cfg)
         return macs
 
@@ -188,6 +229,14 @@ class OFARunner(Runner):
         self,
         subnet_cfg: dict,
     ) -> float:
+        """Estimate the latency of the subnet with the predictor.
+
+        Args:
+            subnet_cfg (dict): The dictionary describing the subnet.
+
+        Returns:
+            Latency of the subnet.
+        """
         latency = self.latency_predictor.predict(subnet_cfg)
         return latency
 
@@ -195,6 +244,14 @@ class OFARunner(Runner):
         self,
         subnet_cfg: dict,
     ) -> float: # pragma: no cover
+        """Validate the top1 accuracy of the subnet on the dataset.
+
+        Args:
+            subnet_cfg (dict): The dictionary describing the subnet.
+
+        Returns:
+            Top1 accuracy of the subnet.
+        """
         subnet = self.get_subnet(subnet_cfg)
         folder_name = '.torch/tmp-{}'.format(uuid.uuid1().hex)
         run_manager = RunManager(
@@ -203,7 +260,8 @@ class OFARunner(Runner):
         run_manager.reset_running_statistics(net=subnet)
 
         # Test sampled subnet
-        self.run_config.data_provider.assign_active_img_size(subnet_cfg['r'][0])
+        self.run_config.data_provider.assign_active_img_size(
+            subnet_cfg['r'][0])
         loss, acc = run_manager.validate(net=subnet, no_logs=False)
         top1 = acc[0]
         return top1
@@ -212,17 +270,18 @@ class OFARunner(Runner):
         self,
         subnet_cfg: dict,
     ) -> float:
-        """Measure Torch model's FLOPs/MACs as per FVCore calculation
-        Args:
-            subnet_cfg: sub-network Torch model
-        Returns:
-            `macs`
-        """
+        """Measure subnet's FLOPs/MACs as per FVCore calculation.
 
+        Args:
+            subnet_cfg (dict): The dictionary describing the subnet.
+
+        Returns:
+            MACs of the subnet.
+        """
         model = self.get_subnet(subnet_cfg)
         input_size = (self.batch_size, 3, 224, 224)
         macs = get_macs(model=model, input_size=input_size, device=self.device)
-        logger.info('Model\'s macs: {}'.format(macs))
+        logger.info('[DyNAS-T] Model\'s macs: {}'.format(macs))
         return macs
 
     @torch.no_grad()
@@ -232,11 +291,13 @@ class OFARunner(Runner):
         warmup_steps: int = None,
         measure_steps: int = None,
     ) -> Tuple[float, float]:
-        """Measure OFA model's latency.
+        """Measure subnet's latency.
+
         Args:
-            subnet_cfg: sub-network Torch model
+            subnet_cfg (dict): The dictionary describing the subnet.
+
         Returns:
-            mean latency; std latency
+            mean latency; std latency.
         """
         model = self.get_subnet(subnet_cfg)
         input_size = (self.batch_size, 3, 224, 224)
@@ -248,7 +309,8 @@ class OFARunner(Runner):
             measure_steps=measure_steps,
             device=self.device,
         )
-        logger.info('Model\'s latency: {} +/- {}'.format(latency_mean, latency_std))
+        logger.info(
+            '[DyNAS-T] Model\'s latency: {} +/- {}'.format(latency_mean, latency_std))
 
         return latency_mean, latency_std
 
@@ -256,6 +318,14 @@ class OFARunner(Runner):
         self,
         subnet_cfg: dict,
     ) -> torch.nn.Module:
+        """Get subnet.
+
+        Args:
+            subnet_cfg (dict): The dictionary describing the subnet.
+
+        Returns:
+            The subnet.
+        """
         if self.supernet == 'ofa_resnet50':
             self.ofa_network.set_active_subnet(
                 ks=subnet_cfg['d'], e=subnet_cfg['e'], d=subnet_cfg['w']
@@ -270,18 +340,105 @@ class OFARunner(Runner):
         return self.subnet
 
 
+class TransformerLTRunner(Runner):  #noqa: D101
+
+    def __init__(
+        self,
+        supernet: str,
+        acc_predictor: Predictor,
+        macs_predictor: Predictor,
+        latency_predictor: Predictor,
+        datasetpath: str,
+        batch_size: int,
+        checkpoint_path: str,
+        **kwargs,
+    ) -> None:  #noqa: D107
+        self.supernet = supernet
+        self.acc_predictor = acc_predictor
+        self.macs_predictor = macs_predictor
+        self.latency_predictor = latency_predictor
+        self.device = 'cpu'
+        self.test_size = None
+        self.batch_size = batch_size
+        self.dataset_path = datasetpath
+        self.checkpoint_path = checkpoint_path
+
+    def estimate_accuracy_bleu(
+        self,
+        subnet_cfg: dict,
+    ) -> float:  #noqa: D102
+        top1 = self.acc_predictor.predict(subnet_cfg)
+        return top1
+
+    def estimate_macs(
+        self,
+        subnet_cfg: dict,
+    ) -> int:  #noqa: D102
+        macs = self.macs_predictor.predict(subnet_cfg)
+        return macs
+
+    def estimate_latency(
+        self,
+        subnet_cfg: dict,
+    ) -> float:  #noqa: D102
+        latency = self.latency_predictor.predict(subnet_cfg)
+        return latency
+
+    def validate_bleu(
+        self,
+        subnet_cfg: dict,
+    ) -> float:    #noqa: D102
+
+        bleu = transformer_interface.compute_bleu(subnet_cfg, self.dataset_path,
+                            self.checkpoint_path)
+        return bleu
+
+    def validate_macs(
+        self,
+        subnet_cfg: dict,
+    ) -> float:
+        """Measure Torch model's FLOPs/MACs as per FVCore calculation.
+
+        Args:
+            subnet_cfg: sub-network Torch model
+        Returns:
+            `macs`
+        """
+        macs = transformer_interface.compute_macs(subnet_cfg, self.dataset_path)
+        logger.info('[DyNAS-T] Model\'s macs: {}'.format(macs))
+
+        return macs
+
+    @torch.no_grad()
+    def measure_latency(
+        self,
+        subnet_cfg: dict,
+    ) -> Tuple[float, float]:
+        """Measure model's latency.
+
+        Args:
+            subnet_cfg: sub-network Torch model
+        Returns:
+            mean latency; std latency
+        """
+        latency_mean, latency_std = transformer_interface.compute_latency(
+            subnet_cfg, self.dataset_path, self.batch_size)
+        logger.info(
+            '[DyNAS-T] Model\'s latency: {} +/- {}'.format(latency_mean, latency_std))
+
+        return latency_mean, latency_std
+
+
 class EvaluationInterface:
-    """
+    """Evaluation Interface class.
+
     The interface class update is required to be updated for each unique SuperNetwork
-    framework as it controls how evaluation calls are made from DyNAS-T
+    framework as it controls how evaluation calls are made from DyNAS-T.
 
     Args:
-        evaluator : class
-            The 'runner' that performs the validation or prediction
-        manager : class
-            The DyNAS-T manager that translates between PyMoo and the parameter dict
-        csv_path : string
-            (Optional) The csv file that get written to during the subnetwork search
+        evaluator (class): The 'runner' that performs the validation or prediction.
+        manager (class): The DyNAS-T manager that translates between PyMoo and the parameter dict.
+        csv_path (str, Optional): The csv file that get written to during the subnetwork search.
     """
 
     def __init__(
@@ -292,6 +449,7 @@ class EvaluationInterface:
         predictor_mode: bool = False,
         csv_path: str = None,
     ) -> None:
+        """Initialize the attributes."""
         self.evaluator = evaluator
         self.manager = manager
         self.metrics = metrics
@@ -302,18 +460,29 @@ class EvaluationInterface:
         self,
         x: list,
     ) -> Tuple[dict, float, float]:
+        """Evaluate the subnet."""
         pass
 
     def clear_csv(self) -> None:
+        """Clear the csv file."""
         if self.csv_path:
             f = open(self.csv_path, "w")
             writer = csv.writer(f)
-            result = ['Sub-network', 'Date', 'Latency (ms)', ' MACs', 'Top-1 Acc (%)']
+            result = ['Sub-network', 'Date',
+                      'Latency (ms)', 'MACs', 'Top-1 Acc (%)']
             writer.writerow(result)
             f.close()
 
 
 class EvaluationInterfaceResNet50(EvaluationInterface):
+    """Evaluation Interface class for ResNet50.
+
+    Args:
+        evaluator (class): The 'runner' that performs the validation or prediction.
+        manager (class): The DyNAS-T manager that translates between PyMoo and the parameter dict.
+        csv_path (str, Optional): The csv file that get written to during the subnetwork search.
+    """
+
     def __init__(
         self,
         evaluator: Runner,
@@ -322,12 +491,14 @@ class EvaluationInterfaceResNet50(EvaluationInterface):
         predictor_mode: bool = False,
         csv_path: str = None,
     ) -> None:
+        """Initialize the attributes."""
         super().__init__(evaluator, manager, metrics, predictor_mode, csv_path)
 
     def eval_subnet(
         self,
         x: list,
     ) -> Tuple[dict, float, float]:
+        """Evaluate the subnet."""
         # PyMoo vector to Elastic Parameter Mapping
         param_dict = self.manager.translate2param(x)
 
@@ -375,6 +546,14 @@ class EvaluationInterfaceResNet50(EvaluationInterface):
 
 
 class EvaluationInterfaceMobileNetV3(EvaluationInterface):
+    """Evaluation Interface class for MobileNetV3.
+
+    Args:
+        evaluator (class): The 'runner' that performs the validation or prediction.
+        manager (class): The DyNAS-T manager that translates between PyMoo and the parameter dict.
+        csv_path (str, Optional): The csv file that get written to during the subnetwork search.
+    """
+
     def __init__(
         self,
         evaluator: Runner,
@@ -383,12 +562,14 @@ class EvaluationInterfaceMobileNetV3(EvaluationInterface):
         predictor_mode=False,
         csv_path=None,
     ) -> None:
+        """Initialize the attributes."""
         super().__init__(evaluator, manager, metrics, predictor_mode, csv_path)
 
     def eval_subnet(
         self,
         x: list,
     ) -> Tuple[dict, float, float]:
+        """Evaluate the subnet."""
         # PyMoo vector to Elastic Parameter Mapping
         param_dict = self.manager.translate2param(x)
 
@@ -404,11 +585,14 @@ class EvaluationInterfaceMobileNetV3(EvaluationInterface):
         # Always evaluate/predict top1
         lat, macs = 0, 0
         if self.predictor_mode == True:
-            top1 = self.evaluator.estimate_accuracy_top1(self.manager.onehot_generic(x).reshape(1,-1))[0]
+            top1 = self.evaluator.estimate_accuracy_top1(
+                self.manager.onehot_generic(x).reshape(1, -1))[0]
             if 'macs' in self.metrics:
-                macs = self.evaluator.estimate_macs(self.manager.onehot_generic(x).reshape(1,-1))[0]
+                macs = self.evaluator.estimate_macs(
+                    self.manager.onehot_generic(x).reshape(1, -1))[0]
             if 'lat' in self.metrics:
-                lat = self.evaluator.estimate_latency(self.manager.onehot_generic(x).reshape(1,-1))[0]
+                lat = self.evaluator.estimate_latency(
+                    self.manager.onehot_generic(x).reshape(1, -1))[0]
         else:
             top1 = self.evaluator.validate_top1(subnet_sample)
             macs = self.evaluator.validate_macs(subnet_sample)
@@ -430,21 +614,108 @@ class EvaluationInterfaceMobileNetV3(EvaluationInterface):
             return sample, macs, -top1
 
 
+class EvaluationInterfaceTransformerLT(EvaluationInterface):  #noqa: D101
+    def __init__(
+        self,
+        evaluator: Runner,
+        manager: ParameterManager,
+        metrics=['acc', 'macs'],
+        predictor_mode=False,
+        csv_path=None,
+    ) -> None:  #noqa: D107
+        super().__init__(evaluator, manager, metrics, predictor_mode, csv_path)
+
+    def eval_subnet(
+        self,
+        x: list,
+    ) -> Tuple[dict, float, float]:  #noqa: D102
+        # PyMoo vector to Elastic Parameter Mapping
+        param_dict = self.manager.translate2param(x)
+
+        sample = {
+            'encoder': {
+                'encoder_embed_dim': param_dict['encoder_embed_dim'][0],
+                'encoder_layer_num': 6,  # param_dict['encoder_layer_num'][0],
+                'encoder_ffn_embed_dim': param_dict['encoder_ffn_embed_dim'],
+                'encoder_self_attention_heads': param_dict['encoder_self_attention_heads'],
+            },
+            'decoder': {
+                'decoder_embed_dim': param_dict['decoder_embed_dim'][0],
+                'decoder_layer_num': param_dict['decoder_layer_num'][0],
+                'decoder_ffn_embed_dim': param_dict['decoder_ffn_embed_dim'],
+                'decoder_self_attention_heads': param_dict['decoder_self_attention_heads'],
+                'decoder_ende_attention_heads': param_dict['decoder_ende_attention_heads'],
+                'decoder_arbitrary_ende_attn': param_dict['decoder_arbitrary_ende_attn']
+            }
+        }
+
+        subnet_sample = copy.deepcopy(sample)
+
+        # Always evaluate/predict top1
+        lat, macs = 0, 0
+        if self.predictor_mode == True:
+            bleu = self.evaluator.estimate_accuracy_bleu(
+                self.manager.onehot_custom(param_dict).reshape(1, -1))[0]
+            if 'macs' in self.metrics:
+                macs = self.evaluator.estimate_macs(
+                    self.manager.onehot_custom(param_dict).reshape(1, -1))[0]
+            if 'lat' in self.metrics:
+                lat = self.evaluator.estimate_latency(
+                    self.manager.onehot_custom(param_dict).reshape(1, -1))[0]
+        else:
+            bleu = self.evaluator.validate_bleu(subnet_sample)
+            macs = self.evaluator.validate_macs(subnet_sample)
+            if 'lat' in self.metrics:
+                lat, _ = self.evaluator.measure_latency(subnet_sample)
+
+        if self.csv_path:
+            with open(self.csv_path, 'a') as f:
+                writer = csv.writer(f)
+                date = str(datetime.now())
+                result = [param_dict, date, lat, macs, bleu, ]
+                writer.writerow(result)
+
+        # PyMoo only minimizes objectives, thus accuracy needs to be negative
+        # Requires format: subnetwork, objective x, objective y
+        if 'lat' in self.metrics:
+            return sample, lat, -bleu
+        else:
+            return sample, macs, -bleu
+
+    def clear_csv(self) -> None:  #noqa: D102
+        if self.csv_path:
+            f = open(self.csv_path, "w")
+            writer = csv.writer(f)
+            result = ['Sub-network', 'Date',
+                      'Latency (ms)', 'MACs', 'BLEU']
+            writer.writerow(result)
+            f.close()
+
+
 def get_torchvision_model(
     model_name: str,
 ) -> torch.nn.Module:
+    """Get the torchvision model.
+
+    Args:
+        model_name (str): The name of the torchvision model.
+
+    Returns:
+        The specified torch model.
+    """
     try:
         model = getattr(torchvision.models, model_name)(pretrained=True)
         model.eval()
         return model
-    except AttributeError as ae: # pragma: no cover
+    except AttributeError as ae:  # pragma: no cover
         logger.error(
             'Model {model_name} not available. This can be due to either a typo or the model is not '
             'available in torchvision=={torchvision_version}. \nAvailable models: {available_models}'.format(
                 model_name=model_name,
                 torchvision_version=torchvision.__version__,
                 available_models=', '.join(
-                    [m for m in dir(torchvision.models) if not m.startswith('_')]
+                    [m for m in dir(torchvision.models)
+                     if not m.startswith('_')]
                 ),
             )
         )
@@ -452,13 +723,25 @@ def get_torchvision_model(
 
 
 class TorchVisionReference:
+    """Baseline of the torchvision model.
+
+    Args:
+        model_name (str): The name of the torchvision model.
+        dataset_path (str): The path to the dataset.
+        batch_size (int): Batch size of the input.
+        input_size (int): Input image's width and height.
+        num_workers (int): How many subprocesses to use for data loading.
+    """
+
     def __init__(
         self,
         model_name: str,
         dataset_path: str,
         batch_size: int,
         input_size: int = 224,
+        num_workers: int = 20,
     ) -> None:
+        """Initialize the attributes."""
         if 'ofa_resnet50' in model_name:
             model_name = 'resnet50'
         if 'ofa_mbv3' in model_name:
@@ -468,6 +751,7 @@ class TorchVisionReference:
         self.dataset_path = dataset_path
         self.batch_size = batch_size
         self.input_size = input_size
+        self.num_workers = num_workers
 
         logger.info(
             '{name} for \'{model_name}\' on \'{val_dataset_path}\' dataset'.format(
@@ -482,9 +766,14 @@ class TorchVisionReference:
         get_torchvision_model(model_name=self.model_name)
 
     def validate_top1(self) -> Tuple[float, float, float]: # pragma: no cover
+        """Get the top1 accuracy of the model on the dataset.
+
+        Returns:
+            Top1 accuracy of the model.
+        """
         ImagenetDataProvider.DEFAULT_PATH = self.dataset_path
         model = get_torchvision_model(model_name=self.model_name)
-        run_config = ImagenetRunConfig(test_batch_size=64, n_worker=20)
+        run_config = ImagenetRunConfig(test_batch_size=64, n_worker=self.num_workers)
         folder_name = '.torch/tmp-{}'.format(uuid.uuid1().hex)
         run_manager = RunManager(
             '{}/eval_subnet'.format(folder_name), model, run_config, init=False
@@ -498,6 +787,11 @@ class TorchVisionReference:
         self,
         device: str = 'cpu',
     ) -> int:
+        """Get the MACs of the model.
+
+        Returns:
+            MACs of the model.
+        """
         model = get_torchvision_model(model_name=self.model_name)
         input_size = (self.batch_size, 3, self.input_size, self.input_size)
 
@@ -516,6 +810,11 @@ class TorchVisionReference:
         warmup_steps: int = None,
         measure_steps: int = None,
     ) -> Tuple[float, float]:
+        """Measure the latency of the model.
+
+        Returns:
+            Latency of the model.
+        """
         model = get_torchvision_model(model_name=self.model_name)
         input_size = (self.batch_size, 3, self.input_size, self.input_size)
 
