@@ -1,3 +1,4 @@
+"""pruning module."""
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
@@ -16,7 +17,7 @@
 # limitations under the License.
 
 from .component import Component
-from ..pruners import PRUNERS
+from ..pruner.pruner_legacy import PRUNERS
 from ..utils import logger
 from ..utils.utility import GLOBAL_STATE, MODE
 from ..utils.create_obj_from_config import create_dataloader, create_train_func, create_eval_func
@@ -25,7 +26,8 @@ from ..adaptor import FRAMEWORKS
 from ..conf.config import PruningConf
 from ..conf.pythonic_config import Config
 
-from warnings import warn
+from deprecated import deprecated
+
 
 class Pruning(Component):
     """This is base class of pruning object.
@@ -40,9 +42,14 @@ class Pruning(Component):
             PruningConf class containing accuracy goal, pruning objective and related
             dataloaders etc.
 
+    Attributes:
+        conf: A config dict object. Contains pruning setting parameters.
+        pruners: A list of Pruner object.
+
     """
 
     def __init__(self, conf_fname_or_obj=None):
+        """Initiailize."""
         super(Pruning, self).__init__()
         if isinstance(conf_fname_or_obj, PruningConf):
             self.conf = conf_fname_or_obj
@@ -56,38 +63,54 @@ class Pruning(Component):
         self.pruners = []
         self.callbacks = dict(tf_pruning=TfPruningCallback)
 
+    def update_items_for_all_pruners(self, **kwargs):
+        """Functions which add User-defined arguments to the original configurations.
+
+        The original config of pruning is read from a file. 
+        However, users can still modify configurations by passing key-value arguments in this function.
+        Please note that the key-value arguments' keys are analysable in current configuration.
+        """
+        for item in self.cfg.pruning.approach.weight_compression_pytorch.pruners:
+            for key in kwargs:
+                if hasattr(item, key):
+                    setattr(item, key, kwargs[key])
+
     def _on_train_begin(self, dataloader=None):
-        """ called before training """
+        """Functions called before training."""
         for pruner in self.pruners:
             pruner.on_train_begin(dataloader)
 
     def _on_epoch_begin(self, epoch):
-        """ called on the beginning of epochs"""
+        """Functions called on the beginning of epochs."""
         for pruner in self.pruners:
             pruner.on_epoch_begin(epoch)
 
     def _on_step_begin(self, batch_id):
-        """ called on the beginning of batches"""
+        """Functions called on the beginning of batches."""
         res = []
         for pruner in self.pruners:
             res.append(pruner.on_step_begin(batch_id))
         return res
 
     def _on_before_optimizer_step(self):
-        """ called after gradient computed """
-        """ usually for getting gradients """
+        """Functions called after gradient computed, usually for getting gradients."""
         for pruner in self.pruners:
             pruner.on_before_optimizer_step()
 
+    def _on_after_optimizer_step(self):
+        """Functions called after optimzier step."""
+        for pruner in self.pruners:
+            pruner.on_after_optimizer_step()
+
     def _on_step_end(self):
-        """ called on the end of batches"""
+        """Functions called on the end of batches."""
         res = []
         for pruner in self.pruners:
             res.append(pruner.on_step_end())
         return res
 
     def _on_epoch_end(self):
-        """ called on the end of epochs"""
+        """Functions called on the end of epochs."""
         res = []
         for pruner in self.pruners:
             res.append(pruner.on_epoch_end())
@@ -98,23 +121,36 @@ class Pruning(Component):
         return res
 
     def _on_train_end(self):
-        """ called after training """
+        """Functions called after training."""
         for pruner in self.pruners:
             pruner.on_train_end()
 
-    def _on_after_optimizer_step(self):
-        """ called after optimzier step """
+    def _on_before_eval(self):
+        """Implement at the beginning of evaluation phase."""
         for pruner in self.pruners:
-            pruner.on_after_optimizer_step()
+            pruner.on_before_eval()
+
+    def _on_after_eval(self):
+        """Implement at the end of evaluation phase."""
+        for pruner in self.pruners:
+            pruner.on_after_eval()
+
+    def prepare(self):
+        """Functions prepare for generate_hooks, generate_pruners."""
+        self.generate_hooks()
+        self.generate_pruners()
 
     def pre_process(self):
+        """Functions called before pruning begins, usually set up pruners."""
         assert isinstance(self._model, BaseModel), 'need set neural_compressor Model for pruning....'
 
         GLOBAL_STATE.STATE = MODE.PRUNING
         framework_specific_info = {'device': self.cfg.device,
                                    'random_seed': self.cfg.tuning.random_seed,
                                    'workspace_path': self.cfg.tuning.workspace.path,
-                                   'q_dataloader': None}
+                                   'q_dataloader': None,
+                                   'format': 'default',
+                                   'backend': 'default'}
 
         if self.framework == 'tensorflow':
             framework_specific_info.update(
@@ -122,8 +158,7 @@ class Pruning(Component):
 
         self.adaptor = FRAMEWORKS[self.framework](framework_specific_info)
 
-        self.generate_hooks()
-        self.generate_pruners()
+        self.prepare()
 
         if self._train_dataloader is None and self._train_func is None:
             train_dataloader_cfg = self.cfg.pruning.train.dataloader
@@ -167,6 +202,10 @@ class Pruning(Component):
             self.register_hook('on_train_begin', self.adaptor._pre_hook_for_hvd)
 
     def execute(self):
+        """Functions that execute the pruning process.
+
+        Follow the working flow: evaluate the dense model -> train/prune the model, evaluate the sparse model.
+        """
         logger.info("Start to get the baseline model's score before pruning.")
         self.baseline_score = self._eval_func(self._model if getattr(self._eval_func, 'builtin', None) \
                         else self._model.model)
@@ -181,7 +220,7 @@ class Pruning(Component):
         return self._model
 
     def generate_hooks(self):
-        # register hooks for pruning
+        """Register hooks for pruning."""
         self.register_hook('on_train_begin', self._on_train_begin)
         self.register_hook('on_train_end', self._on_train_end)
         self.register_hook('on_epoch_begin', self._on_epoch_begin)
@@ -189,8 +228,12 @@ class Pruning(Component):
         self.register_hook('on_step_begin', self._on_step_begin)
         self.register_hook('on_step_end', self._on_step_end)
         self.register_hook('on_before_optimizer_step', self._on_before_optimizer_step)
+        self.register_hook('on_after_optimizer_step', self._on_after_optimizer_step)
+        self.register_hook('on_before_eval', self._on_before_eval)
+        self.register_hook('on_after_eval', self._on_after_eval)
 
     def generate_pruners(self):
+        """Functions that generate pruners and set up self.pruners."""
         for name in self.cfg.pruning.approach:
             assert name == 'weight_compression' or name == "weight_compression_pytorch", \
                 'now we only support weight_compression and weight_compression_pytorch'
@@ -198,8 +241,10 @@ class Pruning(Component):
             if self.cfg.pruning.approach.weight_compression_pytorch != None:
                 from .pytorch_pruner.pruning import Pruning as PytorchPruning
                 self.pytorch_pruner = PytorchPruning(self.cfg)
-                self.pruners.append(self.pytorch_pruner)
-
+                self.pytorch_pruner.model = self.model._model # extract their pytorch model
+                self.pytorch_pruner.prepare()
+                self.pytorch_pruner.on_train_begin()
+                self.pruners += self.pytorch_pruner.pruners
 
             if self.cfg.pruning.approach.weight_compression != None:
                 for pruner in self.cfg.pruning.approach.weight_compression.pruners:
@@ -228,7 +273,7 @@ class Pruning(Component):
                         assert False, 'now only support {}'.format(PRUNERS.keys())
 
     def __call__(self):
-        """The main entry point of pruning.
+        """Entry point of pruning.
 
            This interface currently only works on pytorch
            and provides three usages:
@@ -266,15 +311,17 @@ class Pruning(Component):
         """
         return super(Pruning, self).__call__()
 
+    """This makes pruning.fit() equals to pruning()."""
     fit = __call__
 
     @property
     def pruning_func(self):
-        """ not support get pruning_func """
+        """Not support get pruning_func."""
         assert False, 'Should not try to get the value of `pruning_func` attribute.'
         return None
 
     @pruning_func.setter
+    @deprecated(version='2.0', reason="please use `train_func` instead")
     def pruning_func(self, user_pruning_func):
         """Training function for pruning.
 
@@ -286,46 +333,94 @@ class Pruning(Component):
                          set eval_dataloader with metric configured or directly eval_func
                          to make evaluation of the model executed.
         """
-        warn('This method is deprecated. please use `train_func` instead.',
-             DeprecationWarning, stacklevel=2)
         self._train_func = user_pruning_func
 
     @property
     def evaluation_distributed(self):
-        """ Getter to know whether need distributed evaluation dataloader"""
+        """Getter to know whether need distributed evaluation dataloader."""
         eval_dataloader_cfg = self.cfg.evaluation.accuracy.dataloader
         yaml_distributed = eval_dataloader_cfg.get('distributed', False)
         return self._evaluation_distributed or yaml_distributed
 
     @evaluation_distributed.setter
     def evaluation_distributed(self, distributed):
+        """Work with the former function."""
         self._evaluation_distributed = distributed
 
     @property
     def train_distributed(self):
-        """ Getter to know whether need distributed training dataloader"""
+        """Getter to know whether need distributed training dataloader."""
         train_dataloader_cfg = self.cfg.pruning.train.dataloader
         yaml_distributed = train_dataloader_cfg.get('distributed', False)
         return self._train_distributed or yaml_distributed
 
     @train_distributed.setter
     def train_distributed(self, distributed):
+        """Work with the former function."""
         self._train_distributed = distributed
 
     def __repr__(self):
+        """Return the class's string representation."""
         return 'Pruning'
 
+    def get_sparsity_ratio(self):
+        """Functions that calculate a modules/layers sparsity.
+
+        Returns:
+            Three floats.
+            elementwise_over_matmul_gemm_conv refers to zero elements' ratio in pruning layers.
+            elementwise_over_all refers to zero elements' ratio in all layers in the model.
+            blockwise_over_matmul_gemm_conv refers to all-zero blocks' ratio in pruning layers.
+        """
+        import torch
+        pattern_sparsity_cnt = 0
+        element_sparsity_cnt = 0
+        for pruner in self.pruners:
+            modules = pruner.modules
+            sparsity_ratio = pruner.pattern.get_sparsity_ratio(pruner.masks)
+            cnt = 0
+            for key in modules.keys():
+                cnt += modules[key].weight.numel()
+            pattern_sparsity_cnt += int(cnt * sparsity_ratio)
+            for key in pruner.masks.keys():
+                element_sparsity_cnt += torch.sum(pruner.masks[key] == 0).data.item()
+
+        linear_conv_cnt = 0
+        param_cnt = 0
+        for name, module in self.model.named_modules():
+            if type(module).__name__ in ["Linear"] or "Conv" in type(module).__name__:
+                linear_conv_cnt += module.weight.numel()
+
+        for n, param in self.model.named_parameters():
+            param_cnt += param.numel()
+        blockwise_over_matmul_gemm_conv = float(pattern_sparsity_cnt) / linear_conv_cnt
+        elementwise_over_matmul_gemm_conv = float(element_sparsity_cnt) / linear_conv_cnt
+        elementwise_over_all = float(
+            element_sparsity_cnt) / param_cnt
+
+        return elementwise_over_matmul_gemm_conv, elementwise_over_all, blockwise_over_matmul_gemm_conv
+
 
 class TfPruningCallback(object):
+    """Class that contains callback functions.
+
+    Args:
+        nc_model: A neural compression model object.
+        hooks: A dict. Contains pure-defined hooks.
+    """
+
     def __init__(self, nc_model, input_model, hooks):
+        """Initialize."""
         self.hooks = hooks
         self.nc_model = nc_model
         self.model = input_model
 
     def __getitem__(self, func):
+        """Get the class's function."""
         return getattr(self, func)
 
     def _set_weights(self):
+        """Copy the input model's weight to the nc_model."""
         res = {}
         for index, layer in enumerate(self.model.layers):
             if len(layer.weights):
@@ -333,26 +428,30 @@ class TfPruningCallback(object):
         self.nc_model.weights = res
 
     def on_train_begin(self, logs=None, dataloader=None):
+        """Call the same-name function from hooks."""
         self.hooks['on_train_begin'](dataloader)
 
     def on_train_end(self, logs=None):
+        """Call the same-name function from hooks."""
         self.hooks['on_train_end']()
 
+    @deprecated(version='2.0', reason="please use `on_train_begin` instead")
     def pre_epoch_begin(self, logs=None, dataloader=None):  # pragma: no cover
-        warn('This method is deprecated. please use `on_train_begin` instead.',
-             DeprecationWarning, stacklevel=2)
+        """Call the same-name function from hooks."""
         self.on_train_begin(logs, dataloader)
 
+    @deprecated(version='2.0', reason="please use `on_train_end` instead")
     def post_epoch_end(self, logs=None):  # pragma: no cover
-        warn('This method is deprecated. please use `on_train_end` instead.',
-             DeprecationWarning, stacklevel=2)
+        """Call the same-name function from hooks."""
         self.on_train_end(logs)
 
     def on_epoch_begin(self, epoch, logs=None):
+        """Call the same-name function from hooks."""
         self._set_weights()
         self.hooks['on_epoch_begin'](epoch)
 
     def on_epoch_end(self, logs=None):
+        """Call the same-name function from hooks."""
         self._set_weights()
         res = self.hooks['on_epoch_end']()
         for layer_index, weights in res[0][0].items():
@@ -361,6 +460,7 @@ class TfPruningCallback(object):
             self.model.layers[layer_index].set_weights(get_weights)
 
     def on_step_begin(self, batch, logs=None):
+        """Call the same-name function from hooks."""
         self._set_weights()
         res = self.hooks['on_step_begin'](batch)
         for layer_index, weights in res[0][0].items():
@@ -368,15 +468,17 @@ class TfPruningCallback(object):
             get_weights[0] = weights
             self.model.layers[layer_index].set_weights(get_weights)
 
+    @deprecated(version='2.0', reason="please use `on_step_begin` instead")
     def on_batch_begin(self, batch, logs=None):  # pragma: no cover
-        warn('This method is deprecated. please use `on_step_begin` instead.',
-             DeprecationWarning, stacklevel=2)
+        """Call the same-name function from hooks."""
         self.on_step_begin(batch, logs)
 
     def on_after_compute_loss(self, input, s_outputs, s_loss, t_outputs=None):
+        """Call the same-name function from hooks."""
         return self.hooks['on_after_compute_loss'](input, s_outputs, s_loss, t_outputs)
 
     def on_step_end(self, logs=None):
+        """Call the same-name function from hooks."""
         self._set_weights()
         res = self.hooks['on_step_end']()
         for layer_index, weights in res[0][0].items():
@@ -384,7 +486,7 @@ class TfPruningCallback(object):
             get_weights[0] = weights
             self.model.layers[layer_index].set_weights(get_weights)
 
+    @deprecated(version='2.0', reason="please use `on_step_end` instead")
     def on_batch_end(self, logs=None):  # pragma: no cover
-        warn('This method is deprecated. please use `on_step_end` instead.',
-             DeprecationWarning, stacklevel=2)
+        """Call the same-name function from hooks."""
         self.on_step_end(logs)
